@@ -2,12 +2,10 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-// This file is loaded into the browser window scope.
-/* eslint-env mozilla/browser-window */
-
 const {
   FX_MONITOR_OAUTH_CLIENT_ID,
   FX_RELAY_OAUTH_CLIENT_ID,
+  SCOPE_APP_SYNC,
   VPN_OAUTH_CLIENT_ID,
 } = ChromeUtils.importESModule(
   "resource://gre/modules/FxAccountsCommon.sys.mjs"
@@ -504,8 +502,10 @@ var gSync = {
   // Returns true if FxA is configured, but the send tab targets list isn't
   // ready yet.
   get sendTabConfiguredAndLoading() {
+    const state = UIState.get();
     return (
-      UIState.get().status == UIState.STATUS_SIGNED_IN &&
+      state.status == UIState.STATUS_SIGNED_IN &&
+      state.syncEnabled &&
       !fxAccounts.device.recentDeviceList
     );
   },
@@ -530,8 +530,10 @@ var gSync = {
 
   getSendTabTargets() {
     const targets = [];
+    const state = UIState.get();
     if (
-      UIState.get().status != UIState.STATUS_SIGNED_IN ||
+      state.status != UIState.STATUS_SIGNED_IN ||
+      !state.syncEnabled ||
       !fxAccounts.device.recentDeviceList
     ) {
       return targets;
@@ -801,7 +803,7 @@ var gSync = {
         this.openPrefsFromFxaMenu("sync_settings", button);
         break;
       case "PanelUI-fxa-menu-setup-sync-button":
-        this.openChooseWhatToSync("sync_settings", button);
+        this.openSyncSetup("sync_settings", button);
         break;
 
       case "PanelUI-fxa-menu-sendtab-connect-device-button":
@@ -833,7 +835,7 @@ var gSync = {
         this.openVPNLink(button);
         break;
       case "PanelUI-fxa-menu-sendtab-not-configured-button":
-        this.openPrefsFromFxaMenu("send_tab", button);
+        this.openSyncSetup("send_tab", button);
         break;
     }
   },
@@ -919,6 +921,7 @@ var gSync = {
 
   /**
    * Potential network call. Fetch the list of OAuth clients attached to the current Mozilla account.
+   *
    * @returns {Promise<boolean>} - Resolves to true if successful, false otherwise.
    */
   async fetchListOfOAuthClients() {
@@ -952,8 +955,8 @@ var gSync = {
   },
 
   showSendToDeviceViewFromFxaMenu(anchor) {
-    const { status } = UIState.get();
-    if (status === UIState.STATUS_NOT_CONFIGURED) {
+    const state = UIState.get();
+    if (state.status !== UIState.STATUS_SIGNED_IN || !state.syncEnabled) {
       PanelUI.showSubView("PanelUI-fxa-menu-sendtab-not-configured", anchor);
       return;
     }
@@ -1113,10 +1116,12 @@ var gSync = {
     this.enableSendTabIfValidTab();
 
     if (!this.getSendTabTargets().length) {
-      PanelMultiView.getViewNode(
-        document,
-        "PanelUI-fxa-menu-sendtab-button"
-      ).hidden = true;
+      for (const id of [
+        "PanelUI-fxa-menu-sendtab-button",
+        "PanelUI-fxa-menu-sendtab-separator",
+      ]) {
+        PanelMultiView.getViewNode(document, id).hidden = true;
+      }
     }
 
     if (anchor.getAttribute("open") == "true") {
@@ -1190,10 +1195,14 @@ var gSync = {
       document,
       "PanelUI-fxa-menu-setup-sync-container"
     );
-
     const fxaToolbarMenuButton = document.getElementById(
       "fxa-toolbar-menu-button"
     );
+    const syncSetupSeparator = PanelMultiView.getViewNode(
+      document,
+      "PanelUI-set-up-sync-separator"
+    );
+
     let fxaAvatarLabelEl = document.getElementById("fxa-avatar-label");
 
     // Reset FxA/Sync UI elements to default, which is signed out
@@ -1306,10 +1315,17 @@ var gSync = {
           if (this._shouldShowSyncOffIndicator()) {
             fxaToolbarMenuButton?.setAttribute("badge-status", "sync-disabled");
           }
-          // Show the sync element depending on if the user is enrolled or not
           syncSetupEl.removeAttribute("hidden");
         }
 
+        if (state.hasSyncKeys) {
+          cadButtonEl.removeAttribute("hidden");
+          syncSetupSeparator.removeAttribute("hidden");
+        } else {
+          cadButtonEl.setAttribute("hidden", "true");
+          syncSetupSeparator.setAttribute("hidden", "true");
+        }
+
         // Reposition profiles elements
         emptyProfilesButton.remove();
         profilesButton.remove();
@@ -1370,10 +1386,12 @@ var gSync = {
       t => !!BrowserUtils.getShareableURL(t.linkedBrowser.currentURI)
     );
 
-    PanelMultiView.getViewNode(
-      document,
-      "PanelUI-fxa-menu-sendtab-button"
-    ).hidden = !canSendAllURIs;
+    for (const id of [
+      "PanelUI-fxa-menu-sendtab-button",
+      "PanelUI-fxa-menu-sendtab-separator",
+    ]) {
+      PanelMultiView.getViewNode(document, id).hidden = !canSendAllURIs;
+    }
   },
 
   // This is mis-named - it can be used to record any FxA UI telemetry, whether from
@@ -1999,9 +2017,13 @@ var gSync = {
 
     let sendTabsToDevice = document.getElementById("context_sendTabToDevice");
     sendTabsToDevice.disabled = !enabled;
+    let sendTabToDeviceSeparator = document.getElementById(
+      "context_sendTabToDeviceSeparator"
+    );
 
     if (hideItems || !hasASendableURI) {
       sendTabsToDevice.hidden = true;
+      sendTabToDeviceSeparator.hidden = true;
     } else {
       let tabCount = aTargetTab.multiselected
         ? gBrowser.multiSelectedTabsCount
@@ -2011,6 +2033,7 @@ var gSync = {
         JSON.stringify({ tabCount })
       );
       sendTabsToDevice.hidden = false;
+      sendTabToDeviceSeparator.hidden = false;
     }
   },
 
@@ -2047,10 +2070,12 @@ var gSync = {
       "context-sendpagetodevice",
       !hideItems && showSendPage
     );
-    contextMenu.showItem(
+    for (const id of [
       "context-sendlinktodevice",
-      !hideItems && showSendLink
-    );
+      "context-sep-sendlinktodevice",
+    ]) {
+      contextMenu.showItem(id, !hideItems && showSendLink);
+    }
 
     if (!showSendLink && !showSendPage) {
       return false;
@@ -2294,6 +2319,41 @@ var gSync = {
     this.openPrefs(entryPoint, null, { action: "choose-what-to-sync" });
   },
 
+  /**
+   * Opens the appropriate sync setup flow based on whether the user has sync keys.
+   * - If the user has sync keys: opens sync preferences to configure what to sync
+   * - If the user doesn't have sync keys (third-party auth): opens FxA to create password
+   */
+  async openSyncSetup(type, sourceElement, extraParams = {}) {
+    this.emitFxaToolbarTelemetry(type, sourceElement);
+    const entryPoint = this._getEntryPointForElement(sourceElement);
+
+    try {
+      // Check if the user has sync keys
+      const hasKeys = await fxAccounts.keys.hasKeysForScope(SCOPE_APP_SYNC);
+
+      if (hasKeys) {
+        // User has keys - go to prefs to configure what to sync
+        this.openPrefs(entryPoint, null, { action: "choose-what-to-sync" });
+      } else {
+        // User doesn't have keys (third-party auth) - go to FxA to create password
+        // This will request SCOPE_APP_SYNC so FxA knows to generate sync keys
+        if (!(await FxAccounts.canConnectAccount())) {
+          return;
+        }
+        const url = await FxAccounts.config.promiseSetPasswordURI(
+          entryPoint,
+          extraParams
+        );
+        switchToTabHavingURI(url, true, { replaceQueryString: true });
+      }
+    } catch (err) {
+      this.log.error("Failed to determine sync setup flow", err);
+      // Fall back to opening prefs
+      this.openPrefs(entryPoint);
+    }
+  },
+
   openSyncedTabsPanel() {
     let placement = CustomizableUI.getPlacementOfWidget("sync-button");
     let area = placement?.area;
@@ -2421,8 +2481,10 @@ var gSync = {
     }
   },
 
-  /** Checks if the current list of attached clients to the Mozilla account
+  /**
+   * Checks if the current list of attached clients to the Mozilla account
    * has a service associated with the passed in Id
+   *
    *  @param {string} clientId
    *   A known static Id from FxA that identifies the service it's associated with
    *  @returns {boolean}
