@@ -6,14 +6,16 @@ function makeInputStream(aString) {
   let stream = Cc["@mozilla.org/io/string-input-stream;1"].createInstance(
     Ci.nsIStringInputStream
   );
-  stream.data = aString;
+  stream.setByteStringData(aString);
   return stream; // XPConnect will QI this to nsIInputStream for us.
 }
 
 add_task(async function test_remoteWebNavigation_postdata() {
-  let { HttpServer } = ChromeUtils.import("resource://testing-common/httpd.js");
-  let { CommonUtils } = ChromeUtils.import(
-    "resource://services-common/utils.js"
+  let { HttpServer } = ChromeUtils.importESModule(
+    "resource://testing-common/httpd.sys.mjs"
+  );
+  let { CommonUtils } = ChromeUtils.importESModule(
+    "resource://services-common/utils.sys.mjs"
   );
 
   let server = new HttpServer();
@@ -46,7 +48,7 @@ add_task(async function test_remoteWebNavigation_postdata() {
   BrowserTestUtils.removeTab(gBrowser.selectedTab);
 
   await new Promise(resolve => {
-    server.stop(function() {
+    server.stop(function () {
       resolve();
     });
   });
