@@ -42,6 +42,7 @@ async function toggleBookmarksToolbar(visible) {
  * Resizes a browser window to a particular width and height, and
  * waits for it to reach a "steady state" with respect to its overflowing
  * toolbars.
+ *
  * @param win (browser window)
  *        The window to resize.
  * @param width (int)
@@ -66,7 +67,7 @@ async function resizeWindow(win, width, height) {
  * This test ensures that there are no unexpected
  * uninterruptible reflows when resizing windows.
  */
-add_task(async function() {
+add_task(async function () {
   const BOOKMARKS_COUNT = 150;
   const STARTING_WIDTH = 600;
   const STARTING_HEIGHT = 400;
@@ -80,7 +81,8 @@ add_task(async function() {
     guid: PlacesUtils.bookmarks.toolbarGuid,
     children: Array(BOOKMARKS_COUNT)
       .fill("")
-      .map((_, i) => ({ url: `http://test.places.${i}/` })),
+      // eslint-disable-next-line @microsoft/sdl/no-insecure-url
+      .map((_, i) => ({ url: `http://test.places.${i}.x/` })),
   });
 
   let wasCollapsed = gToolbar.collapsed;
@@ -119,7 +121,7 @@ add_task(async function() {
   await resizeWindow(win, STARTING_WIDTH, STARTING_HEIGHT);
 
   await withPerfObserver(
-    async function() {
+    async function () {
       await resizeWindow(win, SMALL_WIDTH, SMALL_HEIGHT);
       await resizeWindow(win, STARTING_WIDTH, STARTING_HEIGHT);
     },
