@@ -177,6 +177,11 @@ add_task(async function test_tab_contextmenu() {
     false,
     "Send tab to device is shown"
   );
+  is(
+    document.getElementById("context_sendTabToDeviceSeparator").hidden,
+    false,
+    "Send tab to device separator is shown"
+  );
   is(
     document.getElementById("context_sendTabToDevice").disabled,
     false,
@@ -199,6 +204,11 @@ add_task(async function test_tab_contextmenu_unconfigured() {
     true,
     "Send tab to device is hidden"
   );
+  is(
+    document.getElementById("context_sendTabToDeviceSeparator").hidden,
+    true,
+    "Send tab to device separator is hidden"
+  );
   is(
     document.getElementById("context_sendTabToDevice").disabled,
     false,
@@ -217,6 +227,11 @@ add_task(async function test_tab_contextmenu_not_sendable() {
     true,
     "Send tab to device is hidden"
   );
+  is(
+    document.getElementById("context_sendTabToDeviceSeparator").hidden,
+    true,
+    "Send tab to device is hidden"
+  );
   is(
     document.getElementById("context_sendTabToDevice").disabled,
     true,
@@ -235,6 +250,11 @@ add_task(async function test_tab_contextmenu_not_synced_yet() {
     true,
     "Send tab to device is hidden"
   );
+  is(
+    document.getElementById("context_sendTabToDeviceSeparator").hidden,
+    true,
+    "Send tab to device separator is hidden"
+  );
   is(
     document.getElementById("context_sendTabToDevice").disabled,
     true,
@@ -253,6 +273,11 @@ add_task(async function test_tab_contextmenu_sync_not_ready_configured() {
     true,
     "Send tab to device is hidden"
   );
+  is(
+    document.getElementById("context_sendTabToDeviceSeparator").hidden,
+    true,
+    "Send tab to device is hidden"
+  );
   is(
     document.getElementById("context_sendTabToDevice").disabled,
     true,
@@ -274,6 +299,11 @@ add_task(async function test_tab_contextmenu_sync_not_ready_other_state() {
     true,
     "Send tab to device is hidden"
   );
+  is(
+    document.getElementById("context_sendTabToDeviceSeparator").hidden,
+    true,
+    "Send tab to device separator is hidden"
+  );
   is(
     document.getElementById("context_sendTabToDevice").disabled,
     false,
@@ -294,6 +324,12 @@ add_task(async function test_tab_contextmenu_fxa_disabled() {
     true,
     "Send tab to device is hidden"
   );
+  updateTabContextMenu(testTab);
+  is(
+    document.getElementById("context_sendTabToDeviceSeparator").hidden,
+    true,
+    "Send tab to device separator is hidden"
+  );
 
   getter.restore();
   [...document.querySelectorAll(".sync-ui-item")].forEach(
