@@ -13,9 +13,8 @@ async function openAndCheckMenu(menu, target) {
   await popupshown;
 
   is(menu.state, "open", `Menu panel (${menu.id}) is open.`);
-  is(
-    menu.getAttribute("touchmode"),
-    "true",
+  ok(
+    menu.hasAttribute("touchmode"),
     `Menu panel (${menu.id}) is in touchmode.`
   );
 
@@ -34,6 +33,39 @@ async function openAndCheckMenu(menu, target) {
   menu.hidePopup();
 }
 
+async function openAndCheckLazyMenu(id, target) {
+  let menu = document.getElementById(id);
+
+  EventUtils.synthesizeNativeTapAtCenter(target);
+  let ev = await BrowserTestUtils.waitForEvent(
+    window,
+    "popupshown",
+    true,
+    e => e.target.id == id
+  );
+  menu = ev.target;
+
+  is(menu.state, "open", `Menu panel (${menu.id}) is open.`);
+  ok(
+    menu.hasAttribute("touchmode"),
+    `Menu panel (${menu.id}) is in touchmode.`
+  );
+
+  menu.hidePopup();
+
+  let popupshown = BrowserTestUtils.waitForEvent(menu, "popupshown");
+  EventUtils.synthesizeMouseAtCenter(target, {});
+  await popupshown;
+
+  is(menu.state, "open", `Menu panel (${menu.id}) is open.`);
+  ok(
+    !menu.hasAttribute("touchmode"),
+    `Menu panel (${menu.id}) is not in touchmode.`
+  );
+
+  menu.hidePopup();
+}
+
 // The customization UI menu is not attached to the document when it is
 // closed and hence requires special attention.
 async function openAndCheckCustomizationUIMenu(target) {
@@ -49,11 +81,7 @@ async function openAndCheckCustomizationUIMenu(target) {
     is(menu.state, "open", `Menu for ${target.id} is open`);
   }
 
-  is(
-    menu.getAttribute("touchmode"),
-    "true",
-    `Menu for ${target.id} is in touchmode.`
-  );
+  ok(menu.hasAttribute("touchmode"), `Menu for ${target.id} is in touchmode.`);
 
   menu.hidePopup();
 
@@ -77,9 +105,9 @@ async function openAndCheckCustomizationUIMenu(target) {
   menu.hidePopup();
 }
 
-// Ensure that we can run touch events properly for windows [10]
-add_task(async function setup() {
-  let isWindows = AppConstants.isPlatformAndVersionAtLeast("win", "10.0");
+// Ensure that we can run touch events properly for windows
+add_setup(async function () {
+  let isWindows = AppConstants.platform == "win";
   await SpecialPowers.pushPrefEnv({
     set: [["apz.test.fails_with_native_injection", isWindows]],
   });
@@ -95,10 +123,15 @@ add_task(async function test_main_menu_touch() {
 // Test the page action menu.
 add_task(async function test_page_action_panel_touch() {
   // The page action menu only appears on a web page.
-  await BrowserTestUtils.withNewTab("https://example.com", async function() {
-    let pageActionPanel = document.getElementById("pageActionPanel");
+  await BrowserTestUtils.withNewTab("https://example.com", async function () {
+    // The page actions button is not normally visible, so we must
+    // unhide it.
+    BrowserPageActions.mainButtonNode.style.display = "flex";
+    registerCleanupFunction(() => {
+      BrowserPageActions.mainButtonNode.style.removeProperty("display");
+    });
     let target = document.getElementById("pageActionButton");
-    await openAndCheckMenu(pageActionPanel, target);
+    await openAndCheckLazyMenu("pageActionPanel", target);
   });
 });
 
@@ -147,13 +180,6 @@ add_task(async function test_overflow_panel_touch() {
 
 // Test the list all tabs menu.
 add_task(async function test_list_all_tabs_touch() {
-  // Force the menu button to be shown.
-  let tabs = document.getElementById("tabbrowser-tabs");
-  if (!tabs.hasAttribute("overflow")) {
-    tabs.setAttribute("overflow", true);
-    registerCleanupFunction(() => tabs.removeAttribute("overflow"));
-  }
-
   let target = document.getElementById("alltabs-button");
   await openAndCheckCustomizationUIMenu(target);
 });
