@@ -12,22 +12,20 @@ var { XPCOMUtils } = ChromeUtils.importESModule(
 );
 
 ChromeUtils.defineESModuleGetters(this, {
+  AboutNewTab: "resource:///modules/AboutNewTab.sys.mjs",
   BrowserUtils: "resource://gre/modules/BrowserUtils.sys.mjs",
+  BrowserWindowTracker: "resource:///modules/BrowserWindowTracker.sys.mjs",
   ContextualIdentityService:
     "resource://gre/modules/ContextualIdentityService.sys.mjs",
   ExtensionSettingsStore:
     "resource://gre/modules/ExtensionSettingsStore.sys.mjs",
+  ExtensionUtils: "resource://gre/modules/ExtensionUtils.sys.mjs",
   PrivateBrowsingUtils: "resource://gre/modules/PrivateBrowsingUtils.sys.mjs",
-  ShellService: "resource:///modules/ShellService.sys.mjs",
+  ShellService: "moz-src:///browser/components/shell/ShellService.sys.mjs",
   URILoadingHelper: "resource:///modules/URILoadingHelper.sys.mjs",
 });
 
-XPCOMUtils.defineLazyModuleGetters(this, {
-  AboutNewTab: "resource:///modules/AboutNewTab.jsm",
-  BrowserWindowTracker: "resource:///modules/BrowserWindowTracker.jsm",
-});
-
-XPCOMUtils.defineLazyGetter(this, "ReferrerInfo", () =>
+ChromeUtils.defineLazyGetter(this, "ReferrerInfo", () =>
   Components.Constructor(
     "@mozilla.org/referrer-info;1",
     "nsIReferrerInfo",
@@ -60,7 +58,7 @@ Object.defineProperty(this, "BROWSER_NEW_TAB_URL", {
       if (
         !privateAllowed &&
         (extensionControlled ||
-          AboutNewTab.newTabURL.startsWith("moz-extension://"))
+          ExtensionUtils.isExtensionUrl(AboutNewTab.newTabURL))
       ) {
         return "about:privatebrowsing";
       }
@@ -110,16 +108,6 @@ function openUILink(
   );
 }
 
-// This is here for historical reasons. bug 1742889 covers cleaning this up.
-function getRootEvent(aEvent) {
-  return BrowserUtils.getRootEvent(aEvent);
-}
-
-// This is here for historical reasons. bug 1742889 covers cleaning this up.
-function whereToOpenLink(e, ignoreButton, ignoreAlt) {
-  return BrowserUtils.whereToOpenLink(e, ignoreButton, ignoreAlt);
-}
-
 function openTrustedLinkIn(url, where, params) {
   URILoadingHelper.openTrustedLinkIn(window, url, where, params);
 }
@@ -165,7 +153,7 @@ function checkForMiddleClick(node, event) {
       event.metaKey,
       0,
       event,
-      event.mozInputSource
+      event.inputSource
     );
     node.dispatchEvent(cmdEvent);
 
@@ -196,24 +184,20 @@ function createUserContextMenu(
     event.target.firstChild.remove();
   }
 
-  let bundle = Services.strings.createBundle(
-    "chrome://browser/locale/browser.properties"
-  );
+  MozXULElement.insertFTLIfNeeded("toolkit/global/contextual-identity.ftl");
   let docfrag = document.createDocumentFragment();
 
   // If we are excluding a userContextId, we want to add a 'no-container' item.
   if (excludeUserContextId || showDefaultTab) {
     let menuitem = document.createXULElement("menuitem");
+    if (useAccessKeys) {
+      document.l10n.setAttributes(menuitem, "user-context-none");
+    } else {
+      const label =
+        ContextualIdentityService.formatContextLabel("user-context-none");
+      menuitem.setAttribute("label", label);
+    }
     menuitem.setAttribute("data-usercontextid", "0");
-    menuitem.setAttribute(
-      "label",
-      bundle.GetStringFromName("userContextNone.label")
-    );
-    menuitem.setAttribute(
-      "accesskey",
-      bundle.GetStringFromName("userContextNone.accesskey")
-    );
-
     if (!isContextMenu) {
       menuitem.setAttribute("command", "Browser:NewUserContextTab");
     }
@@ -231,16 +215,15 @@ function createUserContextMenu(
 
     let menuitem = document.createXULElement("menuitem");
     menuitem.setAttribute("data-usercontextid", identity.userContextId);
-    menuitem.setAttribute(
-      "label",
-      ContextualIdentityService.getUserContextLabel(identity.userContextId)
-    );
-
-    if (identity.accessKey && useAccessKeys) {
-      menuitem.setAttribute(
-        "accesskey",
-        bundle.GetStringFromName(identity.accessKey)
+    if (identity.name) {
+      menuitem.setAttribute("label", identity.name);
+    } else if (useAccessKeys) {
+      document.l10n.setAttributes(menuitem, identity.l10nId);
+    } else {
+      const label = ContextualIdentityService.formatContextLabel(
+        identity.l10nId
       );
+      menuitem.setAttribute("label", label);
     }
 
     menuitem.classList.add("menuitem-iconic");
@@ -259,15 +242,13 @@ function createUserContextMenu(
     docfrag.appendChild(document.createXULElement("menuseparator"));
 
     let menuitem = document.createXULElement("menuitem");
-    menuitem.setAttribute(
-      "label",
-      bundle.GetStringFromName("userContext.aboutPage.label")
-    );
     if (useAccessKeys) {
-      menuitem.setAttribute(
-        "accesskey",
-        bundle.GetStringFromName("userContext.aboutPage.accesskey")
+      document.l10n.setAttributes(menuitem, "user-context-manage-containers");
+    } else {
+      const label = ContextualIdentityService.formatContextLabel(
+        "user-context-manage-containers"
       );
+      menuitem.setAttribute("label", label);
     }
     menuitem.setAttribute("command", "Browser:OpenAboutContainers");
     docfrag.appendChild(menuitem);
@@ -292,7 +273,8 @@ function closeMenus(node) {
   }
 }
 
-/** This function takes in a key element and compares it to the keys pressed during an event.
+/**
+ * This function takes in a key element and compares it to the keys pressed during an event.
  *
  * @param aEvent
  *        The KeyboardEvent event you want to compare against your key.
@@ -303,7 +285,7 @@ function closeMenus(node) {
  *        to check if the close command key was pressed in aEvent.
  */
 function eventMatchesKey(aEvent, aKey) {
-  let keyPressed = aKey.getAttribute("key").toLowerCase();
+  let keyPressed = (aKey.getAttribute("key") || "").toLowerCase();
   let keyModifiers = aKey.getAttribute("modifiers");
   let modifiers = ["Alt", "Control", "Meta", "Shift"];
 
@@ -350,7 +332,7 @@ function gatherTextUnder(root) {
     } else if (HTMLImageElement.isInstance(node)) {
       // If it has an "alt" attribute, add that.
       var altText = node.getAttribute("alt");
-      if (altText && altText != "") {
+      if (altText) {
         text += " " + altText;
       }
     }
@@ -440,8 +422,7 @@ async function openPreferences(paneID, extraArgs) {
       }
     }
   }
-  let preferencesURL =
-    "about:preferences" +
+  let preferencesURLSuffix =
     (params ? "?" + params : "") +
     (friendlyCategoryName ? "#" + friendlyCategoryName : "");
   let newLoad = true;
@@ -453,7 +434,7 @@ async function openPreferences(paneID, extraArgs) {
     let supportsStringPrefURL = Cc[
       "@mozilla.org/supports-string;1"
     ].createInstance(Ci.nsISupportsString);
-    supportsStringPrefURL.data = preferencesURL;
+    supportsStringPrefURL.data = "about:preferences" + preferencesURLSuffix;
     windowArguments.appendElement(supportsStringPrefURL);
 
     win = Services.ww.openWindow(
@@ -467,11 +448,28 @@ async function openPreferences(paneID, extraArgs) {
     let shouldReplaceFragment = friendlyCategoryName
       ? "whenComparingAndReplace"
       : "whenComparing";
-    newLoad = !win.switchToTabHavingURI(preferencesURL, true, {
-      ignoreFragment: shouldReplaceFragment,
-      replaceQueryString: true,
-      triggeringPrincipal: Services.scriptSecurityManager.getSystemPrincipal(),
-    });
+    newLoad = !win.switchToTabHavingURI(
+      "about:settings" + preferencesURLSuffix,
+      false,
+      {
+        ignoreFragment: shouldReplaceFragment,
+        replaceQueryString: true,
+        triggeringPrincipal:
+          Services.scriptSecurityManager.getSystemPrincipal(),
+      }
+    );
+    if (newLoad) {
+      newLoad = !win.switchToTabHavingURI(
+        "about:preferences" + preferencesURLSuffix,
+        true,
+        {
+          ignoreFragment: shouldReplaceFragment,
+          replaceQueryString: true,
+          triggeringPrincipal:
+            Services.scriptSecurityManager.getSystemPrincipal(),
+        }
+      );
+    }
     browser = win.gBrowser.selectedBrowser;
   }
 
@@ -542,35 +540,6 @@ function buildHelpMenu() {
   if (typeof gSafeBrowsing != "undefined") {
     gSafeBrowsing.setReportPhishingMenu();
   }
-
-  // We're testing to see if the WebCompat team's "Report Site Issue"
-  // access point makes sense in the Help menu. Normally checking this
-  // pref wouldn't be enough, since there's also the case that the
-  // add-on has somehow been disabled by the user or third-party software
-  // without flipping the pref. Since this add-on is only used on pre-release
-  // channels, and since the jury is still out on whether or not the Help menu
-  // is the right place for this item, we're going to do a least-effort
-  // approach here and assume that the pref is enough to determine whether the
-  // menuitem should appear.
-  //
-  // See bug 1690573 for further details.
-  let reportSiteIssueEnabled = Services.prefs.getBoolPref(
-    "extensions.webcompat-reporter.enabled",
-    false
-  );
-  let reportSiteIssue = document.getElementById("help_reportSiteIssue");
-  reportSiteIssue.hidden = !reportSiteIssueEnabled;
-  if (reportSiteIssueEnabled) {
-    let uri = gBrowser.currentURI;
-    let isReportablePage =
-      uri && (uri.schemeIs("http") || uri.schemeIs("https"));
-    reportSiteIssue.disabled = !isReportablePage;
-  }
-
-  if (NimbusFeatures.deviceMigration.getVariable("helpMenuHidden")) {
-    let helpMenuItem = document.getElementById("helpSwitchDevice");
-    helpMenuItem.hidden = true;
-  }
 }
 
 function isElementVisible(aElement) {
@@ -594,18 +563,6 @@ function getHelpLinkURL(aHelpTopic) {
   return url + aHelpTopic;
 }
 
-// aCalledFromModal is optional
-function openHelpLink(aHelpTopic, aCalledFromModal, aWhere) {
-  var url = getHelpLinkURL(aHelpTopic);
-  var where = aWhere;
-  if (!aWhere) {
-    where = aCalledFromModal ? "window" : "tab";
-  }
-
-  openTrustedLinkIn(url, where);
-}
-
-function openPrefsHelp(aEvent) {
-  let helpTopic = aEvent.target.getAttribute("helpTopic");
-  openHelpLink(helpTopic);
+function openHelpLink(aHelpTopic) {
+  openTrustedLinkIn(getHelpLinkURL(aHelpTopic), "tab");
 }
