@@ -8,6 +8,9 @@ add_setup(async function () {
     gBrowser,
     url: "about:logins",
   });
+  await SpecialPowers.pushPrefEnv({
+    set: [["toolkit.osKeyStore.unofficialBuildOnlyLogin", ""]],
+  });
   registerCleanupFunction(() => {
     BrowserTestUtils.removeTab(gBrowser.selectedTab);
   });
@@ -58,7 +61,7 @@ add_task(async function test_login_item() {
     await SpecialPowers.spawn(browser, [], async () => {
       let loginList = content.document.querySelector("login-list");
       let loginListItem = loginList.shadowRoot.querySelector(
-        ".login-list-item[data-guid]:not([hidden])"
+        "login-list-item[data-guid]:not([hidden])"
       );
       info("Clicking on the first login");
       loginListItem.click();
@@ -71,12 +74,15 @@ add_task(async function test_login_item() {
       }, "Waiting for login item to get populated");
       Assert.ok(loginItemPopulated, "The login item should get populated");
     });
-    let reauthObserved = OSKeyStoreTestUtils.waitForOSKeyStoreLogin(true);
+    let reauthObserved = Promise.resolve();
+    if (OSKeyStore.canReauth()) {
+      reauthObserved = OSKeyStoreTestUtils.waitForOSKeyStoreLogin(true);
+    }
     await SpecialPowers.spawn(browser, [], async () => {
       let loginItem = Cu.waiveXrays(
         content.document.querySelector("login-item")
       );
-      let editButton = loginItem.shadowRoot.querySelector(".edit-button");
+      let editButton = loginItem.shadowRoot.querySelector("edit-button");
       editButton.click();
     });
     await reauthObserved;
@@ -91,7 +97,7 @@ add_task(async function test_login_item() {
       usernameInput.value += "-undone";
       passwordInput.value += "-undone";
 
-      let deleteButton = loginItem.shadowRoot.querySelector(".delete-button");
+      let deleteButton = loginItem.shadowRoot.querySelector("delete-button");
       deleteButton.click();
 
       let confirmDeleteDialog = Cu.waiveXrays(
@@ -107,7 +113,7 @@ add_task(async function test_login_item() {
     return SpecialPowers.spawn(browser, [], async () => {
       let loginList = content.document.querySelector("login-list");
       let loginListItem = loginList.shadowRoot.querySelector(
-        ".login-list-item[data-guid]:not([hidden])"
+        "login-list-item[data-guid]:not([hidden])"
       );
       info("Clicking on the first login");
       loginListItem.click();
@@ -120,7 +126,7 @@ add_task(async function test_login_item() {
       }, "Waiting for login item to get populated");
       Assert.ok(loginItemPopulated, "The login item should get populated");
 
-      let deleteButton = loginItem.shadowRoot.querySelector(".delete-button");
+      let deleteButton = loginItem.shadowRoot.querySelector("delete-button");
       deleteButton.click();
 
       let confirmDeleteDialog = Cu.waiveXrays(
