@@ -3,8 +3,8 @@
  */
 "use strict";
 
-const { AttributionCode } = ChromeUtils.import(
-  "resource:///modules/AttributionCode.jsm"
+const { AttributionCode } = ChromeUtils.importESModule(
+  "moz-src:///browser/components/attribution/AttributionCode.sys.mjs"
 );
 
 // Keep in sync with `BROWSER_ATTRIBUTION_ERRORS` in Histograms.json.
@@ -12,15 +12,14 @@ const INDEX_READ_ERROR = 0;
 const INDEX_DECODE_ERROR = 1;
 const INDEX_WRITE_ERROR = 2;
 const INDEX_QUARANTINE_ERROR = 3;
+const INDEX_EMPTY_ERROR = 4;
+const INDEX_NULL_ERROR = 5;
 
-add_setup(function() {
+add_setup(function () {
   // AttributionCode._clearCache is only possible in a testing environment
-  let env = Cc["@mozilla.org/process/environment;1"].getService(
-    Ci.nsIEnvironment
-  );
-  env.set("XPCSHELL_TEST_PROFILE_DIR", "testing");
+  Services.env.set("XPCSHELL_TEST_PROFILE_DIR", "testing");
 
   registerCleanupFunction(() => {
-    env.set("XPCSHELL_TEST_PROFILE_DIR", null);
+    Services.env.set("XPCSHELL_TEST_PROFILE_DIR", null);
   });
 });
