@@ -10,26 +10,11 @@
 // modules. All of the code is installed on |global|, which is a scope
 // shared among the different ext-*.js scripts.
 
-ChromeUtils.defineModuleGetter(
-  this,
-  "PrivateBrowsingUtils",
-  "resource://gre/modules/PrivateBrowsingUtils.jsm"
-);
-ChromeUtils.defineModuleGetter(
-  this,
-  "BrowserWindowTracker",
-  "resource:///modules/BrowserWindowTracker.jsm"
-);
-ChromeUtils.defineModuleGetter(
-  this,
-  "PromiseUtils",
-  "resource://gre/modules/PromiseUtils.jsm"
-);
-ChromeUtils.defineModuleGetter(
-  this,
-  "AboutReaderParent",
-  "resource:///actors/AboutReaderParent.jsm"
-);
+ChromeUtils.defineESModuleGetters(this, {
+  AboutReaderParent: "resource:///actors/AboutReaderParent.sys.mjs",
+  BrowserWindowTracker: "resource:///modules/BrowserWindowTracker.sys.mjs",
+  PrivateBrowsingUtils: "resource://gre/modules/PrivateBrowsingUtils.sys.mjs",
+});
 
 var { ExtensionError } = ExtensionUtils;
 
@@ -49,7 +34,6 @@ extensions.on("uninstalling", (msg, extension) => {
   if (extension.uninstallURL) {
     let browser = windowTracker.topWindow.gBrowser;
     browser.addTab(extension.uninstallURL, {
-      disallowInheritPrincipal: true,
       relatedToCurrent: true,
       triggeringPrincipal: Services.scriptSecurityManager.createNullPrincipal(
         {}
@@ -83,8 +67,12 @@ global.openOptionsPage = extension => {
     return Promise.reject({ message: "No browser window available" });
   }
 
-  if (extension.manifest.options_ui.open_in_tab) {
-    window.switchToTabHavingURI(extension.manifest.options_ui.page, true, {
+  const { optionsPageProperties } = extension;
+  if (!optionsPageProperties) {
+    return Promise.reject({ message: "No options page" });
+  }
+  if (optionsPageProperties.open_in_tab) {
+    window.switchToTabHavingURI(optionsPageProperties.page, true, {
       triggeringPrincipal: extension.principal,
     });
     return Promise.resolve();
@@ -94,7 +82,7 @@ global.openOptionsPage = extension => {
     extension.id
   )}/preferences`;
 
-  return window.BrowserOpenAddonsMgr(viewId);
+  return window.BrowserAddonUI.openAddonsMgr(viewId);
 };
 
 global.makeWidgetId = id => {
@@ -124,7 +112,7 @@ global.clickModifiersFromEvent = event => {
 global.waitForTabLoaded = (tab, url) => {
   return new Promise(resolve => {
     windowTracker.addListener("progress", {
-      onLocationChange(browser, webProgress, request, locationURI, flags) {
+      onLocationChange(browser, webProgress, request, locationURI) {
         if (
           webProgress.isTopLevel &&
           browser.ownerGlobal.gBrowser.getTabForBrowser(browser) == tab &&
@@ -138,15 +126,58 @@ global.waitForTabLoaded = (tab, url) => {
   });
 };
 
-global.replaceUrlInTab = (gBrowser, tab, url) => {
-  let loaded = waitForTabLoaded(tab, url);
-  gBrowser.loadURI(url, {
-    flags: Ci.nsIWebNavigation.LOAD_FLAGS_REPLACE_HISTORY,
+global.replaceUrlInTab = (gBrowser, tab, uri) => {
+  let loaded = waitForTabLoaded(tab, uri.spec);
+  gBrowser.loadURI(uri, {
+    loadFlags: Ci.nsIWebNavigation.LOAD_FLAGS_REPLACE_HISTORY,
     triggeringPrincipal: Services.scriptSecurityManager.getSystemPrincipal(), // This is safe from this functions usage however it would be preferred not to dot his.
   });
   return loaded;
 };
 
+// The tabs.Tab.groupId type in the public extension API is an integer,
+// but tabbrowser's tab group ID are strings. This handles the conversion.
+//
+// tabbrowser.addTabGroup() generates the internal tab group ID as follows:
+// internal group id = `${Date.now()}-${Math.round(Math.random() * 100)}`;
+// After dropping the hyphen ("-"), the result can be coerced into a safe
+// integer.
+//
+// As a safeguard, in case the format changes, we fall back to maintaining
+// an internal mapping (that never gets cleaned up).
+// This may change in https://bugzilla.mozilla.org/show_bug.cgi?id=1960104
+const fallbackTabGroupIdMap = new Map();
+let nextFallbackTabGroupId = 1;
+global.getExtTabGroupIdForInternalTabGroupId = groupIdStr => {
+  const parsedTabId = /^(\d{13})-(\d{1,3})$/.exec(groupIdStr);
+  if (parsedTabId) {
+    const groupId = parsedTabId[1] * 1000 + parseInt(parsedTabId[2], 10);
+    if (Number.isSafeInteger(groupId)) {
+      return groupId;
+    }
+  }
+  // Fall back.
+  let fallbackGroupId = fallbackTabGroupIdMap.get(groupIdStr);
+  if (!fallbackGroupId) {
+    fallbackGroupId = nextFallbackTabGroupId++;
+    fallbackTabGroupIdMap.set(groupIdStr, fallbackGroupId);
+  }
+  return fallbackGroupId;
+};
+global.getInternalTabGroupIdForExtTabGroupId = groupId => {
+  if (Number.isSafeInteger(groupId) && groupId >= 1e15) {
+    // 16 digits - this inverts getExtTabGroupIdForInternalTabGroupId.
+    const groupIdStr = `${Math.floor(groupId / 1000)}-${groupId % 1000}`;
+    return groupIdStr;
+  }
+  for (let [groupIdStr, fallbackGroupId] of fallbackTabGroupIdMap) {
+    if (fallbackGroupId === groupId) {
+      return groupIdStr;
+    }
+  }
+  return null;
+};
+
 /**
  * Manages tab-specific and window-specific context data, and dispatches
  * tab select events across all windows.
@@ -177,7 +208,7 @@ global.TabContext = class extends EventEmitter {
    *
    * @param {XULElement|ChromeWindow} keyObject
    *        Browser tab or browser chrome window.
-   * @returns {Object}
+   * @returns {object}
    */
   get(keyObject) {
     if (!this.tabData.has(keyObject)) {
@@ -254,20 +285,6 @@ global.TabContext = class extends EventEmitter {
   }
 };
 
-// This promise is used to wait for the search service to be initialized.
-// None of the code in the WebExtension modules requests that initialization.
-// It is assumed that it is started at some point. That might never happen,
-// e.g. if the application shuts down before the search service initializes.
-XPCOMUtils.defineLazyGetter(global, "searchInitialized", () => {
-  if (Services.search.isInitialized) {
-    return Promise.resolve();
-  }
-  return ExtensionUtils.promiseObserved(
-    "browser-search-service",
-    (_, data) => data == "init-complete"
-  );
-});
-
 class WindowTracker extends WindowTrackerBase {
   addProgressListener(window, listener) {
     window.gBrowser.addTabsProgressListener(listener);
@@ -290,6 +307,9 @@ class WindowTracker extends WindowTrackerBase {
     if (!context.privateBrowsingAllowed) {
       options.private = false;
     }
+    // bug 1983854 - should this only look for windows on the current
+    // workspace?
+    options.allowFromInactiveWorkspace = true;
     return BrowserWindowTracker.getTopWindow(options);
   }
 }
@@ -455,23 +475,35 @@ class TabTracker extends TabTrackerBase {
   }
 
   /**
-   * Sets the opener of `tab` to the ID `openerTab`. Both tabs must be in the
-   * same window, or this function will throw a type error.
+   * Sets the opener of `tab` to the ID `openerTabId`. Both tabs must be in the
+   * same window, or this function will throw an error. if `openerTabId` is `-1`
+   * the opener tab is cleared.
    *
-   * @param {Element} tab The tab for which to set the owner.
-   * @param {Element} openerTab The opener of <tab>.
+   * @param {Element} nativeTab The tab for which to set the owner.
+   * @param {number} openerTabId The openerTabId of <tab>.
    */
-  setOpener(tab, openerTab) {
-    if (tab.ownerDocument !== openerTab.ownerDocument) {
-      throw new Error("Tab must be in the same window as its opener");
+  setOpener(nativeTab, openerTabId) {
+    let nativeOpenerTab = null;
+
+    if (openerTabId > -1) {
+      nativeOpenerTab = tabTracker.getTab(openerTabId);
+      if (nativeTab.ownerDocument !== nativeOpenerTab.ownerDocument) {
+        throw new ExtensionError(
+          "Opener tab must be in the same window as the tab being updated"
+        );
+      }
+    }
+
+    if (nativeTab.openerTab !== nativeOpenerTab) {
+      nativeTab.openerTab = nativeOpenerTab;
+      this.emit("tab-openerTabId", { nativeTab, openerTabId });
     }
-    tab.openerTab = openerTab;
   }
 
   deferredForTabOpen(nativeTab) {
     let deferred = this._deferredTabOpenEvents.get(nativeTab);
     if (!deferred) {
-      deferred = PromiseUtils.defer();
+      deferred = Promise.withResolvers();
       this._deferredTabOpenEvents.set(nativeTab, deferred);
       deferred.promise.then(() => {
         this._deferredTabOpenEvents.delete(nativeTab);
@@ -494,7 +526,7 @@ class TabTracker extends TabTrackerBase {
     let nativeTab = event.target;
 
     switch (event.type) {
-      case "TabOpen":
+      case "TabOpen": {
         let { adoptedTab } = event.detail;
         if (adoptedTab) {
           // This tab is being created to adopt a tab from a different window.
@@ -525,8 +557,9 @@ class TabTracker extends TabTrackerBase {
           });
         }
         break;
+      }
 
-      case "TabClose":
+      case "TabClose": {
         let { adoptedBy } = event.detail;
         if (adoptedBy) {
           // This tab is being closed because it was adopted by a new window.
@@ -538,6 +571,7 @@ class TabTracker extends TabTrackerBase {
           this.emitRemoved(nativeTab, false);
         }
         break;
+      }
 
       case "TabSelect":
         // Because we are delaying calling emitCreated above, we also need to
@@ -565,7 +599,7 @@ class TabTracker extends TabTrackerBase {
   }
 
   /**
-   * @param {Object} message
+   * @param {object} message
    *        The message to handle.
    * @private
    */
@@ -595,8 +629,10 @@ class TabTracker extends TabTrackerBase {
       // by the first MozAfterPaint event. That code handles finally
       // adopting the tab, and clears it from the arguments list in the
       // process, so if we run later than it, we're too late.
-      let adoptedBy = window.gBrowser.tabs[0];
-      this.adopt(adoptedBy, tabToAdopt);
+      if (window.gBrowser.isTab(tabToAdopt)) {
+        let adoptedBy = window.gBrowser.tabs[0];
+        this.adopt(adoptedBy, tabToAdopt);
+      }
     } else {
       for (let nativeTab of window.gBrowser.tabs) {
         this.emitCreated(nativeTab);
@@ -671,7 +707,7 @@ class TabTracker extends TabTrackerBase {
    *
    * @param {NativeTab} nativeTab
    *        The tab element which is being created.
-   * @param {Object} [currentTabSize]
+   * @param {object} [currentTabSize]
    *        The size of the tab element for the currently active tab.
    * @private
    */
@@ -735,6 +771,23 @@ class TabTracker extends TabTrackerBase {
     };
   }
 
+  getBrowserDataForContext(context) {
+    if (["tab", "background"].includes(context.viewType)) {
+      return this.getBrowserData(context.xulBrowser);
+    } else if (["popup", "sidebar"].includes(context.viewType)) {
+      // popups and sidebars are nested inside a browser element
+      // (with url "chrome://browser/content/webext-panels.xhtml")
+      // and so we look for the corresponding topChromeWindow to
+      // determine the windowId the panel belongs to.
+      const chromeWindow =
+        context.xulBrowser?.ownerGlobal?.browsingContext?.topChromeWindow;
+      const windowId = chromeWindow ? windowTracker.getId(chromeWindow) : -1;
+      return { tabId: -1, windowId };
+    }
+
+    return { tabId: -1, windowId: -1 };
+  }
+
   get activeTab() {
     let window = windowTracker.topWindow;
     if (window && window.gBrowser) {
@@ -762,6 +815,10 @@ class Tab extends TabBase {
     return this.nativeTab.soundPlaying;
   }
 
+  get autoDiscardable() {
+    return !this.nativeTab.undiscardable;
+  }
+
   get browser() {
     return this.nativeTab.linkedBrowser;
   }
@@ -871,20 +928,25 @@ class Tab extends TabBase {
     return successor ? tabTracker.getId(successor) : -1;
   }
 
+  get groupId() {
+    const { group } = this.nativeTab;
+    return group ? getExtTabGroupIdForInternalTabGroupId(group.id) : -1;
+  }
+
   /**
    * Converts session store data to an object compatible with the return value
    * of the convert() method, representing that data.
    *
    * @param {Extension} extension
    *        The extension for which to convert the data.
-   * @param {Object} tabData
+   * @param {object} tabData
    *        Session store data for a closed tab, as returned by
    *        `SessionStore.getClosedTabData()`.
    * @param {DOMWindow} [window = null]
    *        The browser window which the tab belonged to before it was closed.
    *        May be null if the window the tab belonged to no longer exists.
    *
-   * @returns {Object}
+   * @returns {object}
    * @static
    */
   static convertFromSessionStoreClosedData(extension, tabData, window = null) {
@@ -935,7 +997,7 @@ class Window extends WindowBase {
   /**
    * Update the geometry of the browser window.
    *
-   * @param {Object} options
+   * @param {object} options
    *        An object containing new values for the window's geometry.
    * @param {integer} [options.left]
    *        The new pixel distance of the left side of the browser window from
@@ -1001,7 +1063,8 @@ class Window extends WindowBase {
   }
 
   get alwaysOnTop() {
-    return this.appWindow.zLevel >= Ci.nsIAppWindow.raisedZ;
+    // We never create alwaysOnTop browser windows.
+    return false;
   }
 
   get isLastFocused() {
@@ -1012,36 +1075,55 @@ class Window extends WindowBase {
     const STATES = {
       [window.STATE_MAXIMIZED]: "maximized",
       [window.STATE_MINIMIZED]: "minimized",
+      [window.STATE_FULLSCREEN]: "fullscreen",
       [window.STATE_NORMAL]: "normal",
     };
-    let state = STATES[window.windowState];
-    if (window.fullScreen) {
-      state = "fullscreen";
-    }
-    return state;
+    return STATES[window.windowState];
   }
 
   get state() {
     return Window.getState(this.window);
   }
 
-  set state(state) {
+  async setState(state) {
     let { window } = this;
-    if (state !== "fullscreen" && window.fullScreen) {
+
+    const expectedState = (function () {
+      switch (state) {
+        case "maximized":
+          return window.STATE_MAXIMIZED;
+        case "minimized":
+        case "docked":
+          return window.STATE_MINIMIZED;
+        case "normal":
+          return window.STATE_NORMAL;
+        case "fullscreen":
+          return window.STATE_FULLSCREEN;
+      }
+      throw new Error(`Unexpected window state: ${state}`);
+    })();
+
+    const initialState = window.windowState;
+    if (expectedState == initialState) {
+      return;
+    }
+
+    // We check for window.fullScreen here to make sure to exit fullscreen even
+    // if DOM and widget disagree on what the state is. This is a speculative
+    // fix for bug 1780876, ideally it should not be needed.
+    if (initialState == window.STATE_FULLSCREEN || window.fullScreen) {
       window.fullScreen = false;
     }
 
-    switch (state) {
-      case "maximized":
+    switch (expectedState) {
+      case window.STATE_MAXIMIZED:
         window.maximize();
         break;
-
-      case "minimized":
-      case "docked":
+      case window.STATE_MINIMIZED:
         window.minimize();
         break;
 
-      case "normal":
+      case window.STATE_NORMAL:
         // Restore sometimes returns the window to its previous state, rather
         // than to the "normal" state, so it may need to be called anywhere from
         // zero to two times.
@@ -1049,20 +1131,39 @@ class Window extends WindowBase {
         if (window.windowState !== window.STATE_NORMAL) {
           window.restore();
         }
-        if (window.windowState !== window.STATE_NORMAL) {
-          // And on OS-X, where normal vs. maximized is basically a heuristic,
-          // we need to cheat.
-          window.sizeToContent();
-        }
         break;
 
-      case "fullscreen":
+      case window.STATE_FULLSCREEN:
         window.fullScreen = true;
         break;
 
       default:
         throw new Error(`Unexpected window state: ${state}`);
     }
+
+    if (window.windowState != expectedState) {
+      // On Linux, sizemode changes are asynchronous. Some of them might not
+      // even happen if the window manager doesn't want to, so wait for a bit
+      // instead of forever for a sizemode change that might not ever happen.
+      const noWindowManagerTimeout = 2000;
+
+      let onSizeModeChange;
+      const promiseExpectedSizeMode = new Promise(resolve => {
+        onSizeModeChange = function () {
+          if (window.windowState == expectedState) {
+            resolve();
+          }
+        };
+        window.addEventListener("sizemodechange", onSizeModeChange);
+      });
+
+      await Promise.any([
+        promiseExpectedSizeMode,
+        new Promise(resolve => setTimeout(resolve, noWindowManagerTimeout)),
+      ]);
+
+      window.removeEventListener("sizemodechange", onSizeModeChange);
+    }
   }
 
   *getTabs() {
@@ -1119,11 +1220,11 @@ class Window extends WindowBase {
    *
    * @param {Extension} extension
    *        The extension for which to convert the data.
-   * @param {Object} windowData
+   * @param {object} windowData
    *        Session store data for a closed window, as returned by
    *        `SessionStore.getClosedWindowData()`.
    *
-   * @returns {Object}
+   * @returns {object}
    * @static
    */
   static convertFromSessionStoreClosedData(extension, windowData) {
@@ -1132,8 +1233,9 @@ class Window extends WindowBase {
       focused: false,
       incognito: false,
       type: "normal", // this is always "normal" for a closed window
-      // Surely this does not actually work?
-      state: this.getState(windowData),
+      // Bug 1781226: we assert "state" is "normal" in tests, but we could use
+      // the "sizemode" property if we wanted.
+      state: "normal",
       alwaysOnTop: false,
     };
 
@@ -1187,6 +1289,12 @@ class TabManager extends TabManagerBase {
   wrapTab(nativeTab) {
     return new Tab(this.extension, nativeTab, tabTracker.getId(nativeTab));
   }
+
+  getWrapper(nativeTab) {
+    if (!nativeTab.ownerGlobal.gBrowserInit.isAdoptingTab()) {
+      return super.getWrapper(nativeTab);
+    }
+  }
 }
 
 class WindowManager extends WindowManagerBase {
