@@ -16,6 +16,8 @@ add_task(async function testBrowserActionMenuResizeBottomArrow() {
 
   let win = await BrowserTestUtils.openNewBrowserWindow();
 
+  info(`Trying to position in (${left}, ${top})`);
+
   win.resizeTo(WIDTH, HEIGHT);
 
   // Sometimes we run into problems on Linux with resizing being asynchronous
@@ -24,6 +26,8 @@ add_task(async function testBrowserActionMenuResizeBottomArrow() {
   for (let i = 0; i < 20; i++) {
     win.moveTo(left, top);
 
+    info(`got (${win.screenX}, ${win.screenY})`);
+
     if (win.screenX == left && win.screenY == top) {
       break;
     }
@@ -37,7 +41,3 @@ add_task(async function testBrowserActionMenuResizeBottomArrow() {
 
   await BrowserTestUtils.closeWindow(win);
 });
-
-add_task(async function testTeardown() {
-  Services.prefs.clearUserPref("toolkit.cosmeticAnimations.enabled");
-});
