@@ -2,11 +2,14 @@
 /* vim: set sts=2 sw=2 et tw=80: */
 "use strict";
 
-add_task(async function() {
+async function testAction(manifest_version) {
+  const action = manifest_version < 3 ? "browser_action" : "action";
   let extension = ExtensionTestUtils.loadExtension({
     manifest: {
-      browser_action: {
+      manifest_version,
+      [action]: {
         default_popup: "popup.html",
+        default_area: "navbar",
         unrecognized_property: "with-a-random-value",
       },
       icons: { 32: "icon.png" },
@@ -20,7 +23,7 @@ add_task(async function() {
       </body></html>
       `,
 
-      "popup.js": function() {
+      "popup.js": function () {
         window.onload = () => {
           browser.runtime.sendMessage("from-popup");
         };
@@ -28,11 +31,25 @@ add_task(async function() {
       "icon.png": imageBuffer,
     },
 
-    background: function() {
+    background: function () {
       browser.runtime.onMessage.addListener(msg => {
         browser.test.assertEq(msg, "from-popup", "correct message received");
         browser.test.sendMessage("popup");
       });
+
+      // Test what api namespace is valid, make sure both are not.
+      let manifest = browser.runtime.getManifest();
+      let { manifest_version } = manifest;
+      browser.test.assertEq(
+        manifest_version == 2,
+        "browserAction" in browser,
+        "browserAction is available"
+      );
+      browser.test.assertEq(
+        manifest_version !== 2,
+        "action" in browser,
+        "action is available"
+      );
     },
   });
 
@@ -40,7 +57,9 @@ add_task(async function() {
   let waitForConsole = new Promise(resolve => {
     SimpleTest.monitorConsole(resolve, [
       {
-        message: /Reading manifest: Warning processing browser_action.unrecognized_property: An unexpected property was found/,
+        message: new RegExp(
+          `Reading manifest: Warning processing ${action}.unrecognized_property: An unexpected property was found`
+        ),
       },
     ]);
   });
@@ -49,12 +68,18 @@ add_task(async function() {
   await extension.startup();
   ExtensionTestUtils.failOnSchemaWarnings(true);
 
+  let widgetGroup = getBrowserActionWidget(extension);
+  ok(widgetGroup.webExtension, "The extension property was set.");
+
   // Do this a few times to make sure the pop-up is reloaded each time.
   for (let i = 0; i < 3; i++) {
     clickBrowserAction(extension);
 
-    let widget = getBrowserActionWidget(extension).forWindow(window);
-    let image = getComputedStyle(widget.node).listStyleImage;
+    let widget = widgetGroup.forWindow(window);
+    let button = widget.node.querySelector(
+      ".unified-extensions-item-action-button"
+    );
+    let image = getComputedStyle(button).listStyleImage;
 
     ok(image.includes("/icon.png"), "The extension's icon is used");
     await extension.awaitMessage("popup");
@@ -66,4 +91,18 @@ add_task(async function() {
 
   SimpleTest.endMonitorConsole();
   await waitForConsole;
+}
+
+add_setup(async function () {
+  await SpecialPowers.pushPrefEnv({
+    set: [["extensions.manifestV3.enabled", true]],
+  });
+});
+
+add_task(async function test_browserAction() {
+  await testAction(2);
+});
+
+add_task(async function test_action() {
+  await testAction(3);
 });
