@@ -1,5 +1,13 @@
 "use strict";
 
+registerCleanupFunction(() => {
+  // Ensure sidebar is hidden after the test
+  if (!document.getElementById("sidebar-box").hidden) {
+    info("Hiding sidebar that was still open at the end of the test");
+    SidebarController.hide({ dismissPanel: true });
+  }
+});
+
 function background() {
   browser.runtime.onConnect.addListener(port => {
     browser.test.assertEq(port.name, "ernie", "port name correct");
