@@ -5,9 +5,6 @@
 SimpleTest.requestCompleteLog();
 
 add_task(async function test_windows_events_not_allowed() {
-  SpecialPowers.pushPrefEnv({
-    set: [["extensions.allowPrivateBrowsingByDefault", false]],
-  });
   let monitor = await startIncognitoMonitorExtension();
 
   function background() {
@@ -86,11 +83,11 @@ add_task(async function test_windows_events_not_allowed() {
     return windowInfo.winId;
   }
 
-  let {
+  const {
     Management: {
       global: { windowTracker },
     },
-  } = ChromeUtils.import("resource://gre/modules/Extension.jsm", null);
+  } = ChromeUtils.importESModule("resource://gre/modules/Extension.sys.mjs");
 
   let currentWindow = window;
   let currentWindowId = windowTracker.getId(currentWindow);
@@ -149,3 +146,77 @@ add_task(async function test_windows_events_not_allowed() {
   await extension.unload();
   await monitor.unload();
 });
+
+add_task(async function test_windows_event_page() {
+  await SpecialPowers.pushPrefEnv({
+    set: [["extensions.eventPages.enabled", true]],
+  });
+
+  let extension = ExtensionTestUtils.loadExtension({
+    useAddonManager: "permanent",
+    manifest: {
+      browser_specific_settings: { gecko: { id: "eventpage@windows" } },
+      background: { persistent: false },
+    },
+    background() {
+      let removed;
+      browser.windows.onCreated.addListener(window => {
+        browser.test.sendMessage("onCreated", window.id);
+      });
+      browser.windows.onRemoved.addListener(wid => {
+        removed = wid;
+        browser.test.sendMessage("onRemoved", wid);
+      });
+      browser.windows.onFocusChanged.addListener(wid => {
+        if (wid != browser.windows.WINDOW_ID_NONE && wid != removed) {
+          browser.test.sendMessage("onFocusChanged", wid);
+        }
+      });
+      browser.test.sendMessage("ready");
+    },
+  });
+
+  const EVENTS = ["onCreated", "onRemoved", "onFocusChanged"];
+
+  await extension.startup();
+  await extension.awaitMessage("ready");
+  for (let event of EVENTS) {
+    assertPersistentListeners(extension, "windows", event, {
+      primed: false,
+    });
+  }
+
+  // test events waken background
+  await extension.terminateBackground();
+  for (let event of EVENTS) {
+    assertPersistentListeners(extension, "windows", event, {
+      primed: true,
+    });
+  }
+
+  let win = await BrowserTestUtils.openNewBrowserWindow();
+
+  await extension.awaitMessage("ready");
+  let windowId = await extension.awaitMessage("onCreated");
+  ok(true, "persistent event woke background");
+  for (let event of EVENTS) {
+    assertPersistentListeners(extension, "windows", event, {
+      primed: false,
+    });
+  }
+  // focus returns the new window
+  let focusedId = await extension.awaitMessage("onFocusChanged");
+  Assert.equal(windowId, focusedId, "new window was focused");
+  await extension.terminateBackground();
+
+  await BrowserTestUtils.closeWindow(win);
+  await extension.awaitMessage("ready");
+  let removedId = await extension.awaitMessage("onRemoved");
+  Assert.equal(windowId, removedId, "window was removed");
+  // focus returns the window focus was passed to
+  focusedId = await extension.awaitMessage("onFocusChanged");
+  Assert.notEqual(windowId, focusedId, "old window was focused");
+
+  await extension.unload();
+  await SpecialPowers.popPrefEnv();
+});
