@@ -11,15 +11,15 @@ const SECOND_BOOKMARK_TITLE = "Second Bookmark Title";
 const bookmarksInfo = [
   {
     title: "firefox",
-    url: "http://example.com",
+    url: "https://example.com",
   },
   {
     title: "rules",
-    url: "http://example.com/2",
+    url: "https://example.com/2",
   },
   {
     title: "yo",
-    url: "http://example.com/2",
+    url: "https://example.com/2",
   },
 ];
 const TEST_URL = "about:mozilla";
@@ -30,7 +30,11 @@ XPCOMUtils.defineLazyPreferenceGetter(
   "privacy.userContext.enabled"
 );
 
-add_setup(async function() {
+add_setup(async function () {
+  await SpecialPowers.pushPrefEnv({
+    set: [["test.wait300msAfterTabSwitch", true]],
+  });
+
   await PlacesUtils.bookmarks.eraseEverything();
 
   let toolbar = document.getElementById("PersonalToolbar");
@@ -55,7 +59,7 @@ let OptionItemExists = (elementId, doc = document) => {
 
   Assert.ok(optionItem, `Context menu contains the menuitem ${elementId}`);
   Assert.ok(
-    BrowserTestUtils.is_visible(optionItem),
+    BrowserTestUtils.isVisible(optionItem),
     `Context menu option ${elementId} is visible`
   );
 };
@@ -64,7 +68,7 @@ let OptionsMatchExpected = (contextMenu, expectedOptionItems) => {
   let idList = [];
   for (let elem of contextMenu.children) {
     if (
-      BrowserTestUtils.is_visible(elem) &&
+      BrowserTestUtils.isVisible(elem) &&
       elem.localName !== "menuseparator"
     ) {
       idList.push(elem.id);
@@ -90,14 +94,62 @@ let checkContextMenu = async (cbfunc, optionItems, doc = document) => {
     children: bookmarksInfo,
   });
 
-  let contextMenu = await cbfunc(bookmark);
+  // Open and check the context menu twice, once with
+  // `browser.tabs.loadBookmarksInTabs` set to true and again with it set to
+  // false.
+  for (let loadBookmarksInNewTab of [true, false]) {
+    info(
+      `Running checkContextMenu: ` + JSON.stringify({ loadBookmarksInNewTab })
+    );
 
-  for (let item of optionItems) {
-    OptionItemExists(item, doc);
+    await SpecialPowers.pushPrefEnv({
+      set: [["browser.tabs.loadBookmarksInTabs", loadBookmarksInNewTab]],
+    });
+
+    // When `loadBookmarksInTabs` is true, the usual placesContext_open:newtab
+    // item is hidden and placesContext_open is shown. The tasks in this test
+    // assume that `loadBookmarksInTabs` is false, so when a caller expects
+    // placesContext_open:newtab to appear but not placesContext_open, add it to
+    // the list of expected items when the pref is set.
+    let expectedOptionItems = [...optionItems];
+    if (
+      loadBookmarksInNewTab &&
+      optionItems.includes("placesContext_open:newtab") &&
+      !optionItems.includes("placesContext_open")
+    ) {
+      expectedOptionItems.push("placesContext_open");
+    }
+
+    // The caller is responsible for opening the menu, via `cbfunc()`.
+    let contextMenu = await cbfunc(bookmark);
+
+    for (let item of expectedOptionItems) {
+      OptionItemExists(item, doc);
+    }
+
+    OptionsMatchExpected(contextMenu, expectedOptionItems);
+
+    // Check the "default" attributes on placesContext_open and
+    // placesContext_open:newtab.
+    if (expectedOptionItems.includes("placesContext_open")) {
+      Assert.equal(
+        doc.getElementById("placesContext_open").getAttribute("default"),
+        loadBookmarksInNewTab ? null : "true",
+        `placesContext_open has the correct "default" attribute when loadBookmarksInTabs = ${loadBookmarksInNewTab}`
+      );
+    }
+    if (expectedOptionItems.includes("placesContext_open:newtab")) {
+      Assert.equal(
+        doc.getElementById("placesContext_open:newtab").getAttribute("default"),
+        loadBookmarksInNewTab ? "true" : null,
+        `placesContext_open:newtab has the correct "default" attribute when loadBookmarksInTabs = ${loadBookmarksInNewTab}`
+      );
+    }
+
+    contextMenu.hidePopup();
   }
 
-  OptionsMatchExpected(contextMenu, optionItems);
-  contextMenu.hidePopup();
+  await SpecialPowers.popPrefEnv();
   await PlacesUtils.bookmarks.eraseEverything();
 };
 
@@ -126,7 +178,7 @@ add_task(async function test_bookmark_contextmenu_contents() {
     );
   }
 
-  await checkContextMenu(async function() {
+  await checkContextMenu(async function () {
     let toolbarBookmark = await PlacesUtils.bookmarks.insert({
       parentGuid: PlacesUtils.bookmarks.toolbarGuid,
       title: "Bookmark Title",
@@ -149,10 +201,10 @@ add_task(async function test_bookmark_contextmenu_contents() {
     return contextMenu;
   }, optionItems);
 
-  let tab;
+  let tabs = [];
   let contextMenuOnContent;
 
-  await checkContextMenu(async function() {
+  await checkContextMenu(async function () {
     info("Check context menu after opening context menu on content");
     const toolbarBookmark = await PlacesUtils.bookmarks.insert({
       parentGuid: PlacesUtils.bookmarks.toolbarGuid,
@@ -161,7 +213,11 @@ add_task(async function test_bookmark_contextmenu_contents() {
     });
 
     info("Open context menu on about:config");
-    tab = await BrowserTestUtils.openNewForegroundTab(gBrowser, "about:config");
+    let tab = await BrowserTestUtils.openNewForegroundTab(
+      gBrowser,
+      "about:config"
+    );
+    tabs.push(tab);
     contextMenuOnContent = document.getElementById("contentAreaContextMenu");
     const popupShownPromiseOnContent = BrowserTestUtils.waitForEvent(
       contextMenuOnContent,
@@ -172,6 +228,7 @@ add_task(async function test_bookmark_contextmenu_contents() {
       type: "contextmenu",
     });
     await popupShownPromiseOnContent;
+    contextMenuOnContent.hidePopup();
 
     info("Check context menu on bookmark");
     const toolbarNode = getToolbarNodeForItemGuid(toolbarBookmark.guid);
@@ -192,10 +249,11 @@ add_task(async function test_bookmark_contextmenu_contents() {
 
   // We need to do a thorough cleanup to avoid leaking the window of
   // 'about:config'.
-  const tabClosed = BrowserTestUtils.waitForTabClosing(tab);
-  contextMenuOnContent.hidePopup();
-  BrowserTestUtils.removeTab(tab);
-  await tabClosed;
+  for (let tab of tabs) {
+    const tabClosed = BrowserTestUtils.waitForTabClosing(tab);
+    BrowserTestUtils.removeTab(tab);
+    await tabClosed;
+  }
 });
 
 add_task(async function test_empty_contextmenu_contents() {
@@ -210,7 +268,7 @@ add_task(async function test_empty_contextmenu_contents() {
     "show-other-bookmarks_PersonalToolbar",
   ];
 
-  await checkContextMenu(async function() {
+  await checkContextMenu(async function () {
     let contextMenu = document.getElementById("placesContext");
     let toolbar = document.querySelector("#PlacesToolbarItems");
     let openToolbarContextMenuPromise = BrowserTestUtils.waitForPopupEvent(
@@ -245,7 +303,7 @@ add_task(async function test_separator_contextmenu_contents() {
     "show-other-bookmarks_PersonalToolbar",
   ];
 
-  await checkContextMenu(async function() {
+  await checkContextMenu(async function () {
     let sep = await PlacesUtils.bookmarks.insert({
       type: PlacesUtils.bookmarks.TYPE_SEPARATOR,
       parentGuid: PlacesUtils.bookmarks.toolbarGuid,
@@ -284,7 +342,7 @@ add_task(async function test_folder_contextmenu_contents() {
     "show-other-bookmarks_PersonalToolbar",
   ];
 
-  await checkContextMenu(async function() {
+  await checkContextMenu(async function () {
     let folder = await PlacesUtils.bookmarks.insert({
       type: PlacesUtils.bookmarks.TYPE_FOLDER,
       parentGuid: PlacesUtils.bookmarks.toolbarGuid,
@@ -322,7 +380,7 @@ add_task(async function test_sidebar_folder_contextmenu_contents() {
 
   await withSidebarTree("bookmarks", async tree => {
     await checkContextMenu(
-      async bookmark => {
+      async () => {
         let folder = await PlacesUtils.bookmarks.insert({
           parentGuid: PlacesUtils.bookmarks.toolbarGuid,
           title: "folder",
@@ -330,9 +388,10 @@ add_task(async function test_sidebar_folder_contextmenu_contents() {
         });
         tree.selectItems([folder.guid]);
 
-        let contextMenu = SidebarUI.browser.contentDocument.getElementById(
-          "placesContext"
-        );
+        let contextMenu =
+          SidebarController.browser.contentDocument.getElementById(
+            "placesContext"
+          );
         let popupShownPromise = BrowserTestUtils.waitForEvent(
           contextMenu,
           "popupshown"
@@ -342,7 +401,7 @@ add_task(async function test_sidebar_folder_contextmenu_contents() {
         return contextMenu;
       },
       optionItems,
-      SidebarUI.browser.contentDocument
+      SidebarController.browser.contentDocument
     );
   });
 });
@@ -362,7 +421,7 @@ add_task(async function test_sidebar_multiple_folders_contextmenu_contents() {
 
   await withSidebarTree("bookmarks", async tree => {
     await checkContextMenu(
-      async bookmark => {
+      async () => {
         let folder1 = await PlacesUtils.bookmarks.insert({
           parentGuid: PlacesUtils.bookmarks.toolbarGuid,
           title: "folder 1",
@@ -375,9 +434,10 @@ add_task(async function test_sidebar_multiple_folders_contextmenu_contents() {
         });
         tree.selectItems([folder1.guid, folder2.guid]);
 
-        let contextMenu = SidebarUI.browser.contentDocument.getElementById(
-          "placesContext"
-        );
+        let contextMenu =
+          SidebarController.browser.contentDocument.getElementById(
+            "placesContext"
+          );
         let popupShownPromise = BrowserTestUtils.waitForEvent(
           contextMenu,
           "popupshown"
@@ -387,7 +447,7 @@ add_task(async function test_sidebar_multiple_folders_contextmenu_contents() {
         return contextMenu;
       },
       optionItems,
-      SidebarUI.browser.contentDocument
+      SidebarController.browser.contentDocument
     );
   });
 });
@@ -419,9 +479,10 @@ add_task(async function test_sidebar_bookmark_contextmenu_contents() {
       async bookmark => {
         tree.selectItems([bookmark.guid]);
 
-        let contextMenu = SidebarUI.browser.contentDocument.getElementById(
-          "placesContext"
-        );
+        let contextMenu =
+          SidebarController.browser.contentDocument.getElementById(
+            "placesContext"
+          );
         let popupShownPromise = BrowserTestUtils.waitForEvent(
           contextMenu,
           "popupshown"
@@ -431,7 +492,7 @@ add_task(async function test_sidebar_bookmark_contextmenu_contents() {
         return contextMenu;
       },
       optionItems,
-      SidebarUI.browser.contentDocument
+      SidebarController.browser.contentDocument
     );
   });
 });
@@ -460,16 +521,17 @@ add_task(async function test_sidebar_bookmark_search_contextmenu_contents() {
       async bookmark => {
         info("Checking bookmark sidebar menu contents in search context");
         // Perform a search first
-        let searchBox = SidebarUI.browser.contentDocument.getElementById(
-          "search-box"
-        );
-        searchBox.value = SECOND_BOOKMARK_TITLE;
-        searchBox.doCommand();
+        let searchBox =
+          SidebarController.browser.contentDocument.getElementById(
+            "search-box"
+          );
+        await setSearch(searchBox, SECOND_BOOKMARK_TITLE);
         tree.selectItems([bookmark.guid]);
 
-        let contextMenu = SidebarUI.browser.contentDocument.getElementById(
-          "placesContext"
-        );
+        let contextMenu =
+          SidebarController.browser.contentDocument.getElementById(
+            "placesContext"
+          );
         let popupShownPromise = BrowserTestUtils.waitForEvent(
           contextMenu,
           "popupshown"
@@ -479,7 +541,7 @@ add_task(async function test_sidebar_bookmark_search_contextmenu_contents() {
         return contextMenu;
       },
       optionItems,
-      SidebarUI.browser.contentDocument
+      SidebarController.browser.contentDocument
     );
   });
 });
@@ -506,7 +568,7 @@ add_task(async function test_library_bookmark_contextmenu_contents() {
     );
   }
 
-  await withLibraryWindow("BookmarksToolbar", async ({ left, right }) => {
+  await withLibraryWindow("BookmarksToolbar", async ({ right }) => {
     await checkContextMenu(
       async bookmark => {
         let contextMenu = right.ownerDocument.getElementById("placesContext");
@@ -544,14 +606,13 @@ add_task(async function test_library_bookmark_search_contextmenu_contents() {
     );
   }
 
-  await withLibraryWindow("BookmarksToolbar", async ({ left, right }) => {
+  await withLibraryWindow("BookmarksToolbar", async ({ right }) => {
     await checkContextMenu(
       async bookmark => {
         info("Checking bookmark library menu contents in search context");
         // Perform a search first
         let searchBox = right.ownerDocument.getElementById("searchFilter");
-        searchBox.value = SECOND_BOOKMARK_TITLE;
-        searchBox.doCommand();
+        await setSearch(searchBox, SECOND_BOOKMARK_TITLE);
 
         let contextMenu = right.ownerDocument.getElementById("placesContext");
         let popupShownPromise = BrowserTestUtils.waitForEvent(
@@ -590,9 +651,10 @@ add_task(async function test_sidebar_mixedselection_contextmenu_contents() {
         });
         tree.selectItems([bookmark.guid, folder.guid]);
 
-        let contextMenu = SidebarUI.browser.contentDocument.getElementById(
-          "placesContext"
-        );
+        let contextMenu =
+          SidebarController.browser.contentDocument.getElementById(
+            "placesContext"
+          );
         let popupShownPromise = BrowserTestUtils.waitForEvent(
           contextMenu,
           "popupshown"
@@ -602,7 +664,7 @@ add_task(async function test_sidebar_mixedselection_contextmenu_contents() {
         return contextMenu;
       },
       optionItems,
-      SidebarUI.browser.contentDocument
+      SidebarController.browser.contentDocument
     );
   });
 });
@@ -624,14 +686,15 @@ add_task(async function test_sidebar_multiple_bookmarks_contextmenu_contents() {
     await checkContextMenu(
       async bookmark => {
         let bookmark2 = await PlacesUtils.bookmarks.insert({
-          url: "http://example.com/",
+          url: "https://example.com/",
           parentGuid: PlacesUtils.bookmarks.toolbarGuid,
         });
         tree.selectItems([bookmark.guid, bookmark2.guid]);
 
-        let contextMenu = SidebarUI.browser.contentDocument.getElementById(
-          "placesContext"
-        );
+        let contextMenu =
+          SidebarController.browser.contentDocument.getElementById(
+            "placesContext"
+          );
         let popupShownPromise = BrowserTestUtils.waitForEvent(
           contextMenu,
           "popupshown"
@@ -641,82 +704,88 @@ add_task(async function test_sidebar_multiple_bookmarks_contextmenu_contents() {
         return contextMenu;
       },
       optionItems,
-      SidebarUI.browser.contentDocument
+      SidebarController.browser.contentDocument
     );
   });
 });
 
 add_task(async function test_sidebar_multiple_links_contextmenu_contents() {
-  let optionItems = [
-    "placesContext_openLinks:tabs",
-    "placesContext_delete_history",
-    "placesContext_copy",
-    "placesContext_createBookmark",
-  ];
+  if (!Services.prefs.getBoolPref("sidebar.revamp", false)) {
+    let optionItems = [
+      "placesContext_openLinks:tabs",
+      "placesContext_delete_history",
+      "placesContext_copy",
+      "placesContext_createBookmark",
+    ];
 
-  await withSidebarTree("history", async tree => {
-    await checkContextMenu(
-      async bookmark => {
-        await PlacesTestUtils.addVisits([
-          "http://example-1.com/",
-          "http://example-2.com/",
-        ]);
-        // Sort by last visited.
-        tree.ownerDocument.getElementById("bylastvisited").doCommand();
-        tree.selectAll();
+    await withSidebarTree("history", async tree => {
+      await checkContextMenu(
+        async () => {
+          await PlacesTestUtils.addVisits([
+            "https://example-1.com/",
+            "https://example-2.com/",
+          ]);
+          // Sort by last visited.
+          tree.ownerDocument.getElementById("bylastvisited").doCommand();
+          tree.selectAll();
 
-        let contextMenu = SidebarUI.browser.contentDocument.getElementById(
-          "placesContext"
-        );
-        let popupShownPromise = BrowserTestUtils.waitForEvent(
-          contextMenu,
-          "popupshown"
-        );
-        synthesizeClickOnSelectedTreeCell(tree, { type: "contextmenu" });
-        await popupShownPromise;
-        return contextMenu;
-      },
-      optionItems,
-      SidebarUI.browser.contentDocument
-    );
-  });
+          let contextMenu =
+            SidebarController.browser.contentDocument.getElementById(
+              "placesContext"
+            );
+          let popupShownPromise = BrowserTestUtils.waitForEvent(
+            contextMenu,
+            "popupshown"
+          );
+          synthesizeClickOnSelectedTreeCell(tree, { type: "contextmenu" });
+          await popupShownPromise;
+          return contextMenu;
+        },
+        optionItems,
+        SidebarController.browser.contentDocument
+      );
+    });
+  }
 });
 
 add_task(async function test_sidebar_mixed_bookmarks_contextmenu_contents() {
-  let optionItems = [
-    "placesContext_delete",
-    "placesContext_cut",
-    "placesContext_copy",
-    "placesContext_paste_group",
-    "placesContext_new:bookmark",
-    "placesContext_new:folder",
-    "placesContext_new:separator",
-  ];
+  if (!Services.prefs.getBoolPref("sidebar.revamp", false)) {
+    let optionItems = [
+      "placesContext_delete",
+      "placesContext_cut",
+      "placesContext_copy",
+      "placesContext_paste_group",
+      "placesContext_new:bookmark",
+      "placesContext_new:folder",
+      "placesContext_new:separator",
+    ];
 
-  await withSidebarTree("bookmarks", async tree => {
-    await checkContextMenu(
-      async bookmark => {
-        let folder = await PlacesUtils.bookmarks.insert({
-          type: PlacesUtils.bookmarks.TYPE_FOLDER,
-          parentGuid: PlacesUtils.bookmarks.toolbarGuid,
-        });
-        tree.selectItems([bookmark.guid, folder.guid]);
+    await withSidebarTree("bookmarks", async tree => {
+      await checkContextMenu(
+        async bookmark => {
+          let folder = await PlacesUtils.bookmarks.insert({
+            type: PlacesUtils.bookmarks.TYPE_FOLDER,
+            parentGuid: PlacesUtils.bookmarks.toolbarGuid,
+          });
+          tree.selectItems([bookmark.guid, folder.guid]);
 
-        let contextMenu = SidebarUI.browser.contentDocument.getElementById(
-          "placesContext"
-        );
-        let popupShownPromise = BrowserTestUtils.waitForEvent(
-          contextMenu,
-          "popupshown"
-        );
-        synthesizeClickOnSelectedTreeCell(tree, { type: "contextmenu" });
-        await popupShownPromise;
-        return contextMenu;
-      },
-      optionItems,
-      SidebarUI.browser.contentDocument
-    );
-  });
+          let contextMenu =
+            SidebarController.browser.contentDocument.getElementById(
+              "placesContext"
+            );
+          let popupShownPromise = BrowserTestUtils.waitForEvent(
+            contextMenu,
+            "popupshown"
+          );
+          synthesizeClickOnSelectedTreeCell(tree, { type: "contextmenu" });
+          await popupShownPromise;
+          return contextMenu;
+        },
+        optionItems,
+        SidebarController.browser.contentDocument
+      );
+    });
+  }
 });
 
 add_task(async function test_library_noselection_contextmenu_contents() {
@@ -728,9 +797,9 @@ add_task(async function test_library_noselection_contextmenu_contents() {
     "placesContext_paste",
   ];
 
-  await withLibraryWindow("BookmarksToolbar", async ({ left, right }) => {
+  await withLibraryWindow("BookmarksToolbar", async ({ right }) => {
     await checkContextMenu(
-      async bookmark => {
+      async () => {
         let contextMenu = right.ownerDocument.getElementById("placesContext");
         let popupShownPromise = BrowserTestUtils.waitForEvent(
           contextMenu,
@@ -750,3 +819,118 @@ add_task(async function test_library_noselection_contextmenu_contents() {
     );
   });
 });
+
+add_task(async function test_private_browsing_window() {
+  // Test the context menu when in a private browsing window.
+
+  let win = await BrowserTestUtils.openNewBrowserWindow({
+    private: true,
+  });
+
+  let optionItems = [
+    "placesContext_open:newtab",
+    // Hidden in private window "placesContext_open:newcontainertab"
+    // Hidden in private window "placesContext_open:newwindow"
+    "placesContext_open:newprivatewindow",
+    "placesContext_show_bookmark:info",
+    "placesContext_deleteBookmark",
+    "placesContext_cut",
+    "placesContext_copy",
+    "placesContext_paste_group",
+    "placesContext_new:bookmark",
+    "placesContext_new:folder",
+    "placesContext_new:separator",
+  ];
+
+  // Test toolbar.
+  await checkContextMenu(
+    async function () {
+      let toolbarBookmark = await PlacesUtils.bookmarks.insert({
+        parentGuid: PlacesUtils.bookmarks.toolbarGuid,
+        title: "Bookmark Title",
+        url: TEST_URL,
+      });
+
+      let toolbarNode = getToolbarNodeForItemGuid(toolbarBookmark.guid, win);
+
+      let contextMenu = win.document.getElementById("placesContext");
+      let popupShownPromise = BrowserTestUtils.waitForEvent(
+        contextMenu,
+        "popupshown"
+      );
+
+      EventUtils.synthesizeMouseAtCenter(
+        toolbarNode,
+        { button: 2, type: "contextmenu" },
+        win
+      );
+      await popupShownPromise;
+      return contextMenu;
+    },
+    [
+      ...optionItems,
+      "placesContext_showAllBookmarks",
+      "toggle_PersonalToolbar",
+      "show-other-bookmarks_PersonalToolbar",
+    ],
+    win.document
+  );
+
+  // Test side bar.
+  await withSidebarTree(
+    "bookmarks",
+    async tree => {
+      await checkContextMenu(
+        async bookmark => {
+          tree.selectItems([bookmark.guid]);
+
+          let contextMenu =
+            win.SidebarController.browser.contentDocument.getElementById(
+              "placesContext"
+            );
+          let popupShownPromise = BrowserTestUtils.waitForEvent(
+            contextMenu,
+            "popupshown"
+          );
+          synthesizeClickOnSelectedTreeCell(tree, { type: "contextmenu" }, win);
+          await popupShownPromise;
+          return contextMenu;
+        },
+        optionItems,
+        win.SidebarController.browser.contentDocument
+      );
+    },
+    win
+  );
+
+  // Test library window opened when using private browsing window.
+  optionItems.splice(
+    optionItems.indexOf("placesContext_show_bookmark:info"),
+    1
+  );
+  optionItems.splice(0, 0, "placesContext_open");
+
+  await withLibraryWindow(
+    "BookmarksToolbar",
+    async ({ right }) => {
+      await checkContextMenu(
+        async bookmark => {
+          let contextMenu = right.ownerDocument.getElementById("placesContext");
+          let popupShownPromise = BrowserTestUtils.waitForEvent(
+            contextMenu,
+            "popupshown"
+          );
+          right.selectItems([bookmark.guid]);
+          synthesizeClickOnSelectedTreeCell(right, { type: "contextmenu" });
+          await popupShownPromise;
+          return contextMenu;
+        },
+        optionItems,
+        right.ownerDocument
+      );
+    },
+    win
+  );
+
+  await BrowserTestUtils.closeWindow(win);
+});
