@@ -17,7 +17,7 @@ if (commonFile) {
 // Put any other stuff relative to this test folder below.
 
 ChromeUtils.defineESModuleGetters(this, {
-  PlacesUIUtils: "resource:///modules/PlacesUIUtils.sys.mjs",
+  PlacesUIUtils: "moz-src:///browser/components/places/PlacesUIUtils.sys.mjs",
 });
 
 // Needed by some test that relies on having an app registered.
@@ -31,17 +31,9 @@ updateAppInfo({
   platformVersion: "",
 });
 
-// Default bookmarks constants.
-const DEFAULT_BOOKMARKS_ON_TOOLBAR = 1;
+// Default bookmarks constant.
 const DEFAULT_BOOKMARKS_ON_MENU = 1;
 
-function checkItemHasAnnotation(guid, name) {
-  return PlacesUtils.promiseItemId(guid).then(id => {
-    let hasAnnotation = PlacesUtils.annotations.itemHasAnnotation(id, name);
-    Assert.ok(hasAnnotation, `Expected annotation ${name}`);
-  });
-}
-
 var createCorruptDB = async function () {
   let dbPath = PathUtils.join(PathUtils.profileDir, "places.sqlite");
   await IOUtils.remove(dbPath);
