@@ -9,9 +9,12 @@
 // inside the button, which allows the text to be highlighted when the user
 // is searching.
 
-const MozButton = customElements.get("button");
-class HighlightableButton extends MozButton {
+/** @import MozInputSearch from "chrome://global/content/elements/moz-input-search.mjs" */
+
+const MozButtonClass = customElements.get("button");
+class HighlightableButton extends MozButtonClass {
   static get inheritedAttributes() {
+    // @ts-expect-error super is MozButton from toolkit/content/widgets/button.js
     return Object.assign({}, super.inheritedAttributes, {
       ".button-text": "text=label,accesskey,crop",
     });
@@ -22,9 +25,14 @@ customElements.define("highlightable-button", HighlightableButton, {
 });
 
 var gSearchResultsPane = {
+  /** @type {string} */
+  query: undefined,
   listSearchTooltips: new Set(),
   listSearchMenuitemIndicators: new Set(),
+  /** @type {MozInputSearch} */
   searchInput: null,
+  /** @type {HTMLDivElement} */
+  searchTooltipContainer: null,
   // A map of DOM Elements to a string of keywords used in search
   // XXX: We should invalidate this cache on `intl:app-locales-changed`
   searchKeywords: new WeakMap(),
@@ -35,32 +43,45 @@ var gSearchResultsPane = {
 
   searchResultsHighlighted: false,
 
+  searchableNodes: new Set([
+    "button",
+    "label",
+    "description",
+    "menulist",
+    "menuitem",
+    "checkbox",
+  ]),
+
   init() {
     if (this.inited) {
       return;
     }
     this.inited = true;
-    this.searchInput = document.getElementById("searchInput");
-    this.searchInput.hidden = !Services.prefs.getBoolPref(
-      "browser.preferences.search"
+    this.searchInput = /** @type {MozInputSearch} */ (
+      document.getElementById("searchInput")
     );
+    this.searchTooltipContainer = /** @type {HTMLDivElement} */ (
+      document.getElementById("search-tooltip-container")
+    );
+
+    window.addEventListener("resize", () => {
+      this._recomputeTooltipPositions();
+    });
+
     if (!this.searchInput.hidden) {
       this.searchInput.addEventListener("input", this);
-      this.searchInput.addEventListener("command", this);
       window.addEventListener("DOMContentLoaded", () => {
-        this.searchInput.focus();
+        this.searchInput.updateComplete.then(() => {
+          this.searchInput.focus();
+        });
         // Initialize other panes in an idle callback.
         window.requestIdleCallback(() => this.initializeCategories());
       });
     }
-    let helpUrl =
-      Services.urlFormatter.formatURLPref("app.support.baseURL") +
-      "preferences";
-    let helpContainer = document.getElementById("need-help");
-    helpContainer.querySelector("a").href = helpUrl;
     ensureScrollPadding();
   },
 
+  /** @param {InputEvent} event */
   async handleEvent(event) {
     // Ensure categories are initialized if idle callback didn't run sooo enough.
     await this.initializeCategories();
@@ -73,8 +94,8 @@ var gSearchResultsPane = {
    */
   fixInputPosition() {
     let innerContainer = document.querySelector(".sticky-inner-container");
-    let width = window.windowUtils.getBoundsWithoutFlushing(innerContainer)
-      .width;
+    let width =
+      window.windowUtils.getBoundsWithoutFlushing(innerContainer).width;
     innerContainer.style.maxWidth = width + "px";
   },
 
@@ -105,18 +126,21 @@ var gSearchResultsPane = {
     if (!this.categoriesInitialized) {
       this.categoriesInitialized = true;
       // Each element of gCategoryInits is a name
-      for (let [, /* name */ category] of gCategoryInits) {
-        if (!category.inited) {
-          await category.init();
-        }
+      for (let category of gCategoryInits.values()) {
+        category.init();
+      }
+      if (document.hasPendingL10nMutations) {
+        await new Promise(r =>
+          document.addEventListener("L10nMutationsFinished", r, { once: true })
+        );
       }
     }
   },
 
   /**
-   * Finds and returns text nodes within node and all descendants
-   * Iterates through all the sibilings of the node object and adds the sibilings
-   * to an array if sibiling is a TEXT_NODE else checks the text nodes with in current node
+   * Finds and returns text nodes within node and all descendants.
+   * Iterates through all the siblings of the node object and adds each sibling to an
+   * array if it's a TEXT_NODE, and otherwise recurses to check text nodes within it.
    * Source - http://stackoverflow.com/questions/10730309/find-all-text-nodes-in-html-page
    *
    * @param Node nodeObject
@@ -144,6 +168,7 @@ var gSearchResultsPane = {
    * We pass in the nodeSizes to tell exactly where highlighting need be done.
    * When creating the range for highlighting, if the nodes are section is split
    * by an access key, it is important to have the size of each of the nodes summed.
+   *
    * @param Array textNodes
    *    List of DOM elements
    * @param Array nodeSizes
@@ -261,11 +286,6 @@ var gSearchResultsPane = {
     // this next search.
     this.removeAllSearchIndicators(window, !query.length);
 
-    // Clear telemetry request if user types very frequently.
-    if (this.telemetryTimer) {
-      clearTimeout(this.telemetryTimer);
-    }
-
     let srHeader = document.getElementById("header-searchResults");
     let noResultsEl = document.getElementById("no-results-message");
     if (this.query) {
@@ -328,7 +348,8 @@ var gSearchResultsPane = {
           child.classList.remove("visually-hidden");
 
           // Show the preceding search-header if one exists.
-          let groupbox = child.closest("groupbox");
+          let groupbox =
+            child.closest("groupbox") || child.closest("[data-category]");
           let groupHeader =
             groupbox && groupbox.querySelector(".search-header");
           if (groupHeader) {
@@ -362,17 +383,6 @@ var gSearchResultsPane = {
         for (let anchorNode of this.listSearchTooltips) {
           this.createSearchTooltip(anchorNode, this.query);
         }
-
-        // Implant search telemetry probe after user stops typing for a while
-        if (this.query.length >= 2) {
-          this.telemetryTimer = setTimeout(() => {
-            Services.telemetry.keyedScalarAdd(
-              "preferences.search_query",
-              this.query,
-              1
-            );
-          }, 1000);
-        }
       }
     } else {
       if (endQuery) {
@@ -397,6 +407,16 @@ var gSearchResultsPane = {
     );
   },
 
+  /**
+   * Determine if the given element is an anchor tag.
+   *
+   * @param {HTMLElement} el The element.
+   * @returns {boolean} Whether or not the element is an anchor tag.
+   */
+  _isAnchor(el) {
+    return (el.prefix === null || el.prefix === "html") && el.localName === "a";
+  },
+
   /**
    * Finding leaf nodes and checking their content for words to search,
    * It is a recursive function
@@ -411,13 +431,18 @@ var gSearchResultsPane = {
     let matchesFound = false;
     if (
       nodeObject.childElementCount == 0 ||
-      nodeObject.tagName == "button" ||
-      nodeObject.tagName == "label" ||
-      nodeObject.tagName == "description" ||
-      nodeObject.tagName == "menulist" ||
-      nodeObject.tagName == "menuitem"
+      (typeof nodeObject.children !== "undefined" &&
+        Array.prototype.every.call(nodeObject.children, this._isAnchor)) ||
+      this.searchableNodes.has(nodeObject.localName) ||
+      (nodeObject.localName?.startsWith("moz-") &&
+        nodeObject.localName !== "moz-input-box")
     ) {
       let simpleTextNodes = this.textNodeDescendants(nodeObject);
+      if (nodeObject.shadowRoot) {
+        simpleTextNodes.push(
+          ...this.textNodeDescendants(nodeObject.shadowRoot)
+        );
+      }
       for (let node of simpleTextNodes) {
         let result = this.highlightMatches(
           [node],
@@ -436,8 +461,9 @@ var gSearchResultsPane = {
       let accessKeyTextNodes = [];
 
       if (
-        nodeObject.tagName == "label" ||
-        nodeObject.tagName == "description"
+        nodeObject.localName == "label" ||
+        nodeObject.localName == "description" ||
+        nodeObject.localName.startsWith("moz-")
       ) {
         accessKeyTextNodes.push(...simpleTextNodes);
       }
@@ -465,7 +491,7 @@ var gSearchResultsPane = {
       // Searching some elements, such as xul:label, store their user-visible text in a "value" attribute.
       // Value will be skipped for menuitem since value in menuitem could represent index number to distinct each item.
       let valueResult =
-        nodeObject.tagName !== "menuitem" && nodeObject.tagName !== "radio"
+        nodeObject.localName !== "menuitem" && nodeObject.localName !== "radio"
           ? this.queryMatchesContent(
               nodeObject.getAttribute("value"),
               searchPhrase
@@ -493,12 +519,14 @@ var gSearchResultsPane = {
       // Creating tooltips for buttons
       if (
         keywordsResult &&
-        (nodeObject.tagName === "button" || nodeObject.tagName == "menulist")
+        (nodeObject instanceof HTMLElement ||
+          nodeObject.localName === "button" ||
+          nodeObject.localName == "menulist")
       ) {
         this.listSearchTooltips.add(nodeObject);
       }
 
-      if (keywordsResult && nodeObject.tagName === "menuitem") {
+      if (keywordsResult && nodeObject.localName === "menuitem") {
         nodeObject.setAttribute("indicator", "true");
         this.listSearchMenuitemIndicators.add(nodeObject);
         let menulist = nodeObject.closest("menulist");
@@ -508,8 +536,8 @@ var gSearchResultsPane = {
       }
 
       if (
-        (nodeObject.tagName == "menulist" ||
-          nodeObject.tagName == "menuitem") &&
+        (nodeObject.localName == "menulist" ||
+          nodeObject.localName == "menuitem") &&
         (labelResult || valueResult || keywordsResult)
       ) {
         nodeObject.setAttribute("highlightable", "true");
@@ -525,7 +553,7 @@ var gSearchResultsPane = {
 
     // Should not search unselected child nodes of a <xul:deck> element
     // except the "historyPane" <xul:deck> element.
-    if (nodeObject.tagName == "deck" && nodeObject.id != "historyPane") {
+    if (nodeObject.localName == "deck" && nodeObject.id != "historyPane") {
       let index = nodeObject.selectedIndex;
       if (index != -1) {
         let result = await this.searchChildNodeIfVisible(
@@ -568,7 +596,7 @@ var gSearchResultsPane = {
     ) {
       result = await this.searchWithinNode(child, searchPhrase);
       // Creating tooltips for menulist element
-      if (result && nodeObject.tagName === "menulist") {
+      if (result && nodeObject.localName === "menulist") {
         this.listSearchTooltips.add(nodeObject);
       }
 
@@ -576,7 +604,7 @@ var gSearchResultsPane = {
       // add it to the list of subitems. The items that don't match the search term
       // will be hidden.
       if (
-        child instanceof Element &&
+        Element.isInstance(child) &&
         (child.classList.contains("featureGate") ||
           child.classList.contains("mozilla-product-item"))
       ) {
@@ -679,21 +707,59 @@ var gSearchResultsPane = {
     // Set tooltipNode property to track corresponded tooltip node.
     anchorNode.tooltipNode = searchTooltip;
     anchorNode.parentElement.classList.add("search-tooltip-parent");
-    anchorNode.parentElement.appendChild(searchTooltip);
+    this.searchTooltipContainer.append(searchTooltip);
 
-    this.calculateTooltipPosition(anchorNode);
+    this._applyTooltipPosition(
+      searchTooltip,
+      this._computeTooltipPosition(anchorNode, searchTooltip)
+    );
   },
 
-  calculateTooltipPosition(anchorNode) {
-    let searchTooltip = anchorNode.tooltipNode;
+  _recomputeTooltipPositions() {
+    let positions = [];
+    for (let anchorNode of this.listSearchTooltips) {
+      let searchTooltip = anchorNode.tooltipNode;
+      if (!searchTooltip) {
+        continue;
+      }
+      let position = this._computeTooltipPosition(anchorNode, searchTooltip);
+      positions.push({ searchTooltip, position });
+    }
+    for (let { searchTooltip, position } of positions) {
+      this._applyTooltipPosition(searchTooltip, position);
+    }
+  },
+
+  _applyTooltipPosition(searchTooltip, position) {
+    searchTooltip.style.left = position.left + "px";
+    searchTooltip.style.top = position.top + "px";
+  },
+
+  _computeTooltipPosition(anchorNode, searchTooltip) {
     // In order to get the up-to-date position of each of the nodes that we're
-    // putting tooltips on, we have to flush layout intentionally, and that
-    // this is the result of a XUL limitation (bug 1363730).
+    // putting tooltips on, we have to flush layout intentionally. Once
+    // menulists don't use XUL layout we can remove this and use plain CSS to
+    // position them, see bug 1363730.
+    let anchorRect = anchorNode.getBoundingClientRect();
+    let tooltipContainerRect =
+      this.searchTooltipContainer.getBoundingClientRect();
     let tooltipRect = searchTooltip.getBoundingClientRect();
-    searchTooltip.style.setProperty(
-      "left",
-      `calc(50% - ${tooltipRect.width / 2}px)`
-    );
+
+    let top = anchorRect.top - tooltipContainerRect.top;
+
+    let left;
+    if (anchorRect.left <= tooltipContainerRect.left + 20) {
+      // Left align on anchors that are close to the side of the main content
+      left = 8;
+    } else {
+      // Center tooltips if their anchor is floating off somewhere else
+      left =
+        anchorRect.left -
+        tooltipContainerRect.left +
+        anchorRect.width / 2 -
+        tooltipRect.width / 2;
+    }
+    return { left, top };
   },
 
   /**
