@@ -8,6 +8,10 @@
 
 // HOME PAGE
 
+ChromeUtils.defineESModuleGetters(this, {
+  ExtensionUtils: "resource://gre/modules/ExtensionUtils.sys.mjs",
+});
+
 /*
  * Preferences:
  *
@@ -34,6 +38,226 @@ const NEW_TAB_KEY = "newTabURL";
 
 const BLANK_HOMEPAGE_URL = "chrome://browser/content/blanktab.html";
 
+// New Prefs UI: we need to check for this setting before registering prefs
+// so that old-style prefs continue working
+if (Services.prefs.getBoolPref("browser.settings-redesign.enabled")) {
+  Preferences.addAll([
+    { id: "browser.newtabpage.activity-stream.showSearch", type: "bool" },
+    {
+      id: "browser.newtabpage.activity-stream.system.showWeather",
+      type: "bool",
+    },
+    { id: "browser.newtabpage.activity-stream.showWeather", type: "bool" },
+    {
+      id: "browser.newtabpage.activity-stream.widgets.system.enabled",
+      type: "bool",
+    },
+    {
+      id: "browser.newtabpage.activity-stream.widgets.enabled",
+      type: "bool",
+    },
+    {
+      id: "browser.newtabpage.activity-stream.widgets.system.lists.enabled",
+      type: "bool",
+    },
+    {
+      id: "browser.newtabpage.activity-stream.widgets.lists.enabled",
+      type: "bool",
+    },
+    {
+      id: "browser.newtabpage.activity-stream.widgets.system.focusTimer.enabled",
+      type: "bool",
+    },
+    {
+      id: "browser.newtabpage.activity-stream.widgets.focusTimer.enabled",
+      type: "bool",
+    },
+    {
+      id: "browser.newtabpage.activity-stream.feeds.topsites",
+      type: "bool",
+    },
+    {
+      id: "browser.newtabpage.activity-stream.topSitesRows",
+      type: "int",
+    },
+    {
+      id: "browser.newtabpage.activity-stream.feeds.system.topstories",
+      type: "bool",
+    },
+    {
+      id: "browser.newtabpage.activity-stream.feeds.section.topstories",
+      type: "bool",
+    },
+    {
+      id: "browser.newtabpage.activity-stream.showSponsoredCheckboxes",
+      type: "bool",
+    },
+    {
+      id: "browser.newtabpage.activity-stream.showSponsoredTopSites",
+      type: "bool",
+    },
+    {
+      id: "browser.newtabpage.activity-stream.showSponsored",
+      type: "bool",
+    },
+    {
+      id: "browser.newtabpage.activity-stream.feeds.section.highlights",
+      type: "bool",
+    },
+    {
+      id: "browser.newtabpage.activity-stream.section.highlights.rows",
+      type: "int",
+    },
+    {
+      id: "browser.newtabpage.activity-stream.section.highlights.includeVisited",
+      type: "bool",
+    },
+    {
+      id: "browser.newtabpage.activity-stream.section.highlights.includeBookmarks",
+      type: "bool",
+    },
+    {
+      id: "browser.newtabpage.activity-stream.section.highlights.includeDownloads",
+      type: "bool",
+    },
+  ]);
+
+  // Search
+  Preferences.addSetting({
+    id: "webSearch",
+    pref: "browser.newtabpage.activity-stream.showSearch",
+  });
+
+  // Weather
+  Preferences.addSetting({
+    id: "showWeather",
+    pref: "browser.newtabpage.activity-stream.system.showWeather",
+  });
+  Preferences.addSetting({
+    id: "weather",
+    pref: "browser.newtabpage.activity-stream.showWeather",
+    deps: ["showWeather"],
+    visible: ({ showWeather }) => showWeather.value,
+  });
+
+  // Widgets: general
+  Preferences.addSetting({
+    id: "widgetsEnabled",
+    pref: "browser.newtabpage.activity-stream.widgets.system.enabled",
+  });
+  Preferences.addSetting({
+    id: "widgets",
+    pref: "browser.newtabpage.activity-stream.widgets.enabled",
+    deps: ["widgetsEnabled"],
+    visible: ({ widgetsEnabled }) => widgetsEnabled.value,
+  });
+
+  // Widgets: lists
+  Preferences.addSetting({
+    id: "listsEnabled",
+    pref: "browser.newtabpage.activity-stream.widgets.system.lists.enabled",
+  });
+  Preferences.addSetting({
+    id: "lists",
+    pref: "browser.newtabpage.activity-stream.widgets.lists.enabled",
+    deps: ["listsEnabled"],
+    visible: ({ listsEnabled }) => listsEnabled.value,
+  });
+
+  // Widgets: timer
+  Preferences.addSetting({
+    id: "timerEnabled",
+    pref: "browser.newtabpage.activity-stream.widgets.system.focusTimer.enabled",
+  });
+  Preferences.addSetting({
+    id: "timer",
+    pref: "browser.newtabpage.activity-stream.widgets.focusTimer.enabled",
+    deps: ["timerEnabled"],
+    visible: ({ timerEnabled }) => timerEnabled.value,
+  });
+
+  // Shortcuts
+  Preferences.addSetting({
+    id: "shortcuts",
+    pref: "browser.newtabpage.activity-stream.feeds.topsites",
+  });
+  Preferences.addSetting({
+    id: "shortcutsRows",
+    pref: "browser.newtabpage.activity-stream.topSitesRows",
+  });
+
+  // Stories
+  Preferences.addSetting({
+    id: "stories",
+    pref: "browser.newtabpage.activity-stream.feeds.section.topstories",
+  });
+
+  // Dependency prefs for sponsored stories visibility
+  Preferences.addSetting({
+    id: "systemTopstories",
+    pref: "browser.newtabpage.activity-stream.feeds.system.topstories",
+  });
+  Preferences.addSetting({
+    id: "sectionTopstories",
+    pref: "browser.newtabpage.activity-stream.feeds.section.topstories",
+  });
+
+  // Support Firefox: sponsored content
+  Preferences.addSetting({
+    id: "supportFirefox",
+    pref: "browser.newtabpage.activity-stream.showSponsoredCheckboxes",
+    deps: ["sponsoredShortcuts", "sponsoredStories"],
+    onUserChange(value, { sponsoredShortcuts, sponsoredStories }) {
+      // When supportFirefox changes, automatically update child preferences to match
+      sponsoredShortcuts.value = !!value;
+      sponsoredStories.value = !!value;
+    },
+  });
+  Preferences.addSetting({
+    id: "topsitesEnabled",
+    pref: "browser.newtabpage.activity-stream.feeds.topsites",
+  });
+  Preferences.addSetting({
+    id: "sponsoredShortcuts",
+    pref: "browser.newtabpage.activity-stream.showSponsoredTopSites",
+    deps: ["topsitesEnabled"],
+    disabled: ({ topsitesEnabled }) => !topsitesEnabled.value,
+  });
+  Preferences.addSetting({
+    id: "sponsoredStories",
+    pref: "browser.newtabpage.activity-stream.showSponsored",
+    deps: ["systemTopstories", "sectionTopstories"],
+    visible: ({ systemTopstories }) => !!systemTopstories.value,
+    disabled: ({ sectionTopstories }) => !sectionTopstories.value,
+  });
+  Preferences.addSetting({
+    id: "supportFirefoxPromo",
+    deps: ["supportFirefox"],
+  });
+
+  // Recent activity
+  Preferences.addSetting({
+    id: "recentActivity",
+    pref: "browser.newtabpage.activity-stream.feeds.section.highlights",
+  });
+  Preferences.addSetting({
+    id: "recentActivityRows",
+    pref: "browser.newtabpage.activity-stream.section.highlights.rows",
+  });
+  Preferences.addSetting({
+    id: "recentActivityVisited",
+    pref: "browser.newtabpage.activity-stream.section.highlights.includeVisited",
+  });
+  Preferences.addSetting({
+    id: "recentActivityBookmarks",
+    pref: "browser.newtabpage.activity-stream.section.highlights.includeBookmarks",
+  });
+  Preferences.addSetting({
+    id: "recentActivityDownloads",
+    pref: "browser.newtabpage.activity-stream.section.highlights.includeDownloads",
+  });
+}
+
 var gHomePane = {
   HOME_MODE_FIREFOX_HOME: "0",
   HOME_MODE_BLANK: "1",
@@ -131,6 +355,7 @@ var gHomePane = {
 
   /**
    *  _updateMenuInterface: adds items to or removes them from the menulists
+   *
    * @param {string} selectId Optional Id of the menulist to add or remove items from.
    *                          If not included this will update both home and newtab menus.
    */
@@ -160,13 +385,14 @@ var gHomePane = {
       }
 
       let extensionOptions;
+      await ExtensionSettingsStore.initialize();
       if (select.id === "homeMode") {
-        extensionOptions = await ExtensionSettingsStore.getAllSettings(
+        extensionOptions = ExtensionSettingsStore.getAllSettings(
           PREF_SETTING_TYPE,
           HOMEPAGE_OVERRIDE_KEY
         );
       } else {
-        extensionOptions = await ExtensionSettingsStore.getAllSettings(
+        extensionOptions = ExtensionSettingsStore.getAllSettings(
           URL_OVERRIDES_TYPE,
           NEW_TAB_KEY
         );
@@ -186,11 +412,9 @@ var gHomePane = {
         );
         if (!currentOption) {
           let option = document.createXULElement("menuitem");
-          option.classList.add("addon-with-favicon");
           option.value = addon.id;
           option.label = addon.name;
           menupopup.append(option);
-          option.querySelector("image").src = addon.iconURL;
         }
         let setting = extensionOptions.find(o => o.id == addon.id);
         if (
@@ -282,6 +506,7 @@ var gHomePane = {
   /**
    * _renderCustomSettings: Hides or shows the UI for setting a custom
    * homepage URL
+   *
    * @param {obj} options
    * @param {bool} options.shouldShow Should the custom UI be shown?
    * @param {bool} options.isControlled Is an extension controlling the home page?
@@ -322,6 +547,7 @@ var gHomePane = {
 
   /**
    * _isHomePageDefaultValue
+   *
    * @returns {bool} Is the homepage set to the default pref value?
    */
   _isHomePageDefaultValue() {
@@ -333,6 +559,7 @@ var gHomePane = {
 
   /**
    * isHomePageBlank
+   *
    * @returns {bool} Is the homepage set to about:blank?
    */
   isHomePageBlank() {
@@ -344,16 +571,21 @@ var gHomePane = {
   },
 
   /**
-   * _isTabAboutPreferences: Is a given tab set to about:preferences?
+   * _isTabAboutPreferencesOrSettings: Is a given tab set to about:preferences or about:settings?
+   *
    * @param {Element} aTab A tab element
-   * @returns {bool} Is the linkedBrowser of aElement set to about:preferences?
+   * @returns {bool} Is the linkedBrowser of aElement set to about:preferences or about:settings?
    */
-  _isTabAboutPreferences(aTab) {
-    return aTab.linkedBrowser.currentURI.spec.startsWith("about:preferences");
+  _isTabAboutPreferencesOrSettings(aTab) {
+    return (
+      aTab.linkedBrowser.currentURI.spec.startsWith("about:preferences") ||
+      aTab.linkedBrowser.currentURI.spec.startsWith("about:settings")
+    );
   },
 
   /**
    * _getTabsForHomePage
+   *
    * @returns {Array} An array of current tabs
    */
   _getTabsForHomePage() {
@@ -366,8 +598,8 @@ var gHomePane = {
       win.document.documentElement.getAttribute("windowtype") ===
         "navigator:browser"
     ) {
-      tabs = win.gBrowser.visibleTabs.slice(win.gBrowser._numPinnedTabs);
-      tabs = tabs.filter(tab => !this._isTabAboutPreferences(tab));
+      tabs = win.gBrowser.visibleTabs.slice(win.gBrowser.pinnedTabCount);
+      tabs = tabs.filter(tab => !this._isTabAboutPreferencesOrSettings(tab));
       // XXX: Bug 1441637 - Fix tabbrowser to report tab.closing before it blurs it
       tabs = tabs.filter(tab => !tab.closing);
     }
@@ -434,7 +666,7 @@ var gHomePane = {
       this._renderCustomSettings();
       this._setInputDisabledStates(false);
     } else {
-      if (HomePage.get().startsWith("moz-extension:")) {
+      if (ExtensionUtils.isExtensionUrl(HomePage.get())) {
         controllingExtension = await getControllingExtension(
           PREF_SETTING_TYPE,
           HOMEPAGE_OVERRIDE_KEY
@@ -654,6 +886,8 @@ var gHomePane = {
   },
 
   init() {
+    initSettingGroup("home");
+
     // Event Listeners
     document
       .getElementById("homePageUrl")
