@@ -2,45 +2,61 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at <http://mozilla.org/MPL/2.0/>. */
 
-import React from "react";
-import { bindActionCreators, combineReducers } from "redux";
-import ReactDOM from "react-dom";
-const { Provider } = require("react-redux");
+import React from "devtools/client/shared/vendor/react";
+import {
+  bindActionCreators,
+  combineReducers,
+} from "devtools/client/shared/vendor/redux";
+import ReactDOM from "devtools/client/shared/vendor/react-dom";
+const {
+  Provider,
+} = require("resource://devtools/client/shared/vendor/react-redux.js");
 
 import ToolboxProvider from "devtools/client/framework/store-provider";
 import flags from "devtools/shared/flags";
+const {
+  registerStoreObserver,
+} = require("resource://devtools/client/shared/redux/subscriber.js");
 
-const { AppConstants } = require("resource://gre/modules/AppConstants.jsm");
+const { AppConstants } = ChromeUtils.importESModule(
+  "resource://gre/modules/AppConstants.sys.mjs"
+);
 
-import * as search from "../workers/search";
-import * as prettyPrint from "../workers/pretty-print";
-import { ParserDispatcher } from "../workers/parser";
+import { SearchDispatcher } from "../workers/search/index";
+import { PrettyPrintDispatcher } from "../workers/pretty-print/index";
 
 import configureStore from "../actions/utils/create-store";
-import reducers from "../reducers";
-import * as selectors from "../selectors";
+import reducers from "../reducers/index";
+import * as selectors from "../selectors/index";
 import App from "../components/App";
 import { asyncStore, prefs } from "./prefs";
-import { persistTabs } from "../utils/tabs";
-const { sanitizeBreakpoints } = require("devtools/client/shared/thread-utils");
+const {
+  sanitizeBreakpoints,
+} = require("resource://devtools/client/shared/thread-utils.js");
+const {
+  visibilityHandlerStore,
+} = require("resource://devtools/client/shared/redux/visibilityHandlerStore.js");
 
-let parser;
+let gWorkers;
 
 export function bootstrapStore(client, workers, panel, initialState) {
   const debugJsModules = AppConstants.DEBUG_JS_MODULES == "1";
   const createStore = configureStore({
     log: prefs.logging || flags.testing,
     timing: debugJsModules,
-    makeThunkArgs: (args, state) => {
-      return { ...args, client, ...workers, panel };
-    },
+    thunkArgs: { client, ...workers, panel },
   });
 
-  const store = createStore(combineReducers(reducers), initialState);
+  let store = createStore(combineReducers(reducers), initialState);
+
+  // Also wrap the store in order to pause store update notifications while the panel is hidden.
+  store = visibilityHandlerStore(store);
+
   registerStoreObserver(store, updatePrefs);
 
   const actions = bindActionCreators(
-    require("../actions").default,
+    // eslint-disable-next-line mozilla/reject-relative-requires
+    require("../actions/index").default,
     store.dispatch
   );
 
@@ -48,20 +64,18 @@ export function bootstrapStore(client, workers, panel, initialState) {
 }
 
 export function bootstrapWorkers(panelWorkers) {
-  const workerPath = "resource://devtools/client/debugger/dist";
-
-  prettyPrint.start(`${workerPath}/pretty-print-worker.js`);
-  parser = new ParserDispatcher();
-
-  parser.start(`${workerPath}/parser-worker.js`);
-  search.start(`${workerPath}/search-worker.js`);
-  return { ...panelWorkers, prettyPrint, parser, search };
+  // The panel worker will typically be the source map and parser workers.
+  // Both will be managed by the toolbox.
+  gWorkers = {
+    prettyPrintWorker: new PrettyPrintDispatcher(),
+    searchWorker: new SearchDispatcher(),
+  };
+  return { ...panelWorkers, ...gWorkers };
 }
 
 export function teardownWorkers() {
-  prettyPrint.stop();
-  parser.stop();
-  search.stop();
+  gWorkers.prettyPrintWorker.stop();
+  gWorkers.searchWorker.stop();
 }
 
 /**
@@ -69,7 +83,7 @@ export function teardownWorkers() {
  *
  * @param {ReduxStore} store
  * @param {ReduxStore} toolboxStore
- * @param {Object} appComponentAttributes
+ * @param {object} appComponentAttributes
  * @param {Array} appComponentAttributes.fluentBundles
  * @param {Document} appComponentAttributes.toolboxDoc
  */
@@ -102,15 +116,6 @@ export function unmountRoot() {
   ReactDOM.unmountComponentAtNode(getMountElement());
 }
 
-function registerStoreObserver(store, subscriber) {
-  let oldState = store.getState();
-  store.subscribe(() => {
-    const state = store.getState();
-    subscriber(state, oldState);
-    oldState = state;
-  });
-}
-
 function updatePrefs(state, oldState) {
   const hasChanged = selector =>
     selector(oldState) && selector(oldState) !== selector(state);
@@ -128,8 +133,14 @@ function updatePrefs(state, oldState) {
     asyncStore.eventListenerBreakpoints = state.eventListenerBreakpoints;
   }
 
-  if (hasChanged(selectors.getTabs)) {
-    asyncStore.tabs = persistTabs(selectors.getTabs(state));
+  if (hasChanged(selectors.getOpenedURLs)) {
+    asyncStore.openedURLs = selectors.getOpenedURLs(state);
+  }
+  if (hasChanged(selectors.getPrettyPrintedURLs)) {
+    // Convert the Set into an Array
+    asyncStore.prettyPrintedURLs = Array.from(
+      selectors.getPrettyPrintedURLs(state)
+    );
   }
 
   if (hasChanged(selectors.getXHRBreakpoints)) {
@@ -139,4 +150,9 @@ function updatePrefs(state, oldState) {
   if (hasChanged(selectors.getBlackBoxRanges)) {
     asyncStore.blackboxedRanges = selectors.getBlackBoxRanges(state);
   }
+
+  if (hasChanged(selectors.getMainThreadProjectDirectoryRoots)) {
+    asyncStore.directoryRoots =
+      selectors.getMainThreadProjectDirectoryRoots(state);
+  }
 }
