@@ -7,10 +7,11 @@
 #include "nsHtml5DocumentBuilder.h"
 
 #include "mozilla/dom/ScriptLoader.h"
-#include "nsIHTMLDocument.h"
-#include "nsIStyleSheetLinkingElement.h"
+#include "mozilla/dom/LinkStyle.h"
 #include "nsNameSpaceManager.h"
-#include "nsStyleLinkElement.h"
+#include "nsNetUtil.h"
+
+using mozilla::dom::LinkStyle;
 
 NS_IMPL_CYCLE_COLLECTION_INHERITED(nsHtml5DocumentBuilder, nsContentSink,
                                    mOwnedElements)
@@ -32,69 +33,66 @@ nsresult nsHtml5DocumentBuilder::Init(mozilla::dom::Document* aDoc,
   return nsContentSink::Init(aDoc, aURI, aContainer, aChannel);
 }
 
-nsHtml5DocumentBuilder::~nsHtml5DocumentBuilder() {}
+nsHtml5DocumentBuilder::~nsHtml5DocumentBuilder() = default;
 
 nsresult nsHtml5DocumentBuilder::MarkAsBroken(nsresult aReason) {
   mBroken = aReason;
   return aReason;
 }
 
-void nsHtml5DocumentBuilder::SetDocumentCharsetAndSource(
-    NotNull<const Encoding*> aEncoding, int32_t aCharsetSource) {
-  if (mDocument) {
-    mDocument->SetDocumentCharacterSetSource(aCharsetSource);
-    mDocument->SetDocumentCharacterSet(aEncoding);
-  }
-}
-
 void nsHtml5DocumentBuilder::UpdateStyleSheet(nsIContent* aElement) {
-  nsCOMPtr<nsIStyleSheetLinkingElement> ssle(do_QueryInterface(aElement));
-  if (!ssle) {
+  auto* linkStyle = LinkStyle::FromNode(*aElement);
+  if (!linkStyle) {
     MOZ_ASSERT(nsNameSpaceManager::GetInstance()->mSVGDisabled,
                "Node didn't QI to style, but SVG wasn't disabled.");
     return;
   }
 
-  // Break out of the doc update created by Flush() to zap a runnable
-  // waiting to call UpdateStyleSheet without the right observer
-  EndDocUpdate();
-
-  if (MOZ_UNLIKELY(!mParser)) {
-    // EndDocUpdate ran stuff that called nsIParser::Terminate()
-    return;
-  }
-
-  ssle->SetEnableUpdates(true);
-
-  auto updateOrError =
-      ssle->UpdateStyleSheet(mRunsToCompletion ? nullptr : this);
+  auto updateOrError = linkStyle->EnableUpdatesAndUpdateStyleSheet(
+      mRunsToCompletion ? nullptr : this);
 
   if (updateOrError.isOk() && updateOrError.unwrap().ShouldBlock() &&
       !mRunsToCompletion) {
     ++mPendingSheetCount;
-    mScriptLoader->AddParserBlockingScriptExecutionBlocker();
+    if (mScriptLoader) {
+      mScriptLoader->AddParserBlockingScriptExecutionBlocker();
+    }
   }
-
-  // Re-open update
-  BeginDocUpdate();
 }
 
 void nsHtml5DocumentBuilder::SetDocumentMode(nsHtml5DocumentMode m) {
   nsCompatibility mode = eCompatibility_NavQuirks;
+  const char* errMsgId = nullptr;
+
   switch (m) {
     case STANDARDS_MODE:
       mode = eCompatibility_FullStandards;
       break;
     case ALMOST_STANDARDS_MODE:
       mode = eCompatibility_AlmostStandards;
+      errMsgId = "errAlmostStandardsDoctypeVerbose";
       break;
     case QUIRKS_MODE:
       mode = eCompatibility_NavQuirks;
+      errMsgId = "errQuirkyDoctypeVerbose";
       break;
   }
-  nsCOMPtr<nsIHTMLDocument> htmlDocument = do_QueryInterface(mDocument);
-  NS_ASSERTION(htmlDocument, "Document didn't QI into HTML document.");
-  htmlDocument->SetCompatibilityMode(mode);
+  mDocument->SetCompatibilityMode(mode);
+
+  if (errMsgId && !mDocument->IsLoadedAsData()) {
+    nsCOMPtr<nsIURI> docURI = mDocument->GetDocumentURI();
+    nsCOMPtr<nsIPrincipal> principal = mDocument->GetPrincipal();
+    if (principal->GetIsNullPrincipal() && !docURI->SchemeIs("data") &&
+        !mozilla::net::SchemeIsHttpOrHttps(docURI)) {
+      // Don't normally warn for null principals. It may well be internal
+      // documents for which the warning is not applicable.
+      return;
+    }
+
+    nsContentUtils::ReportToConsole(
+        nsIScriptError::warningFlag, "HTML_PARSER__DOCTYPE"_ns, mDocument,
+        nsContentUtils::eHTMLPARSER_PROPERTIES, errMsgId);
+  }
 }
 
 // nsContentSink overrides
