@@ -1,19 +1,24 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#ifndef SANDBOX_SRC_TARGET_SERVICES_H__
-#define SANDBOX_SRC_TARGET_SERVICES_H__
+#ifndef SANDBOX_WIN_SRC_TARGET_SERVICES_H_
+#define SANDBOX_WIN_SRC_TARGET_SERVICES_H_
 
-#include "base/macros.h"
+#include "base/containers/span.h"
 #include "sandbox/win/src/sandbox.h"
 #include "sandbox/win/src/win_utils.h"
+#include "third_party/abseil-cpp/absl/types/optional.h"
 
 namespace sandbox {
 
 class ProcessState {
  public:
   ProcessState();
+
+  ProcessState(const ProcessState&) = delete;
+  ProcessState& operator=(const ProcessState&) = delete;
+
   // Returns true if main has been called.
   bool InitCalled() const;
   // Returns true if LowerToken has been called.
@@ -30,7 +35,6 @@ class ProcessState {
 
   ProcessStateInternal process_state_;
   bool csrss_connected_;
-  DISALLOW_COPY_AND_ASSIGN(ProcessState);
 };
 
 // This class is an implementation of the  TargetServices.
@@ -41,15 +45,16 @@ class TargetServicesBase : public TargetServices {
  public:
   TargetServicesBase();
 
-  // Public interface of TargetServices.
+  TargetServicesBase(const TargetServicesBase&) = delete;
+  TargetServicesBase& operator=(const TargetServicesBase&) = delete;
+
+  // Public interface of TargetServices. See comments in sandbox.h.
   ResultCode Init() override;
+  absl::optional<base::span<const uint8_t>> GetDelegateData() override;
   void LowerToken() override;
   ProcessState* GetState() override;
-  ResultCode DuplicateHandle(HANDLE source_handle,
-                             DWORD target_process_id,
-                             HANDLE* target_handle,
-                             DWORD desired_access,
-                             DWORD options) override;
+  ResultCode GetComplexLineBreaks(const WCHAR* text, uint32_t length,
+                                  uint8_t* break_before) final;
 
   // Factory method.
   static TargetServicesBase* GetInstance();
@@ -63,9 +68,8 @@ class TargetServicesBase : public TargetServices {
  private:
   ~TargetServicesBase() {}
   ProcessState process_state_;
-  DISALLOW_COPY_AND_ASSIGN(TargetServicesBase);
 };
 
 }  // namespace sandbox
 
-#endif  // SANDBOX_SRC_TARGET_SERVICES_H__
+#endif  // SANDBOX_WIN_SRC_TARGET_SERVICES_H_
