@@ -9,7 +9,7 @@
     from itertools import groupby
 %>
 
-#[cfg(feature = "gecko")] use crate::gecko_bindings::structs::nsCSSPropertyID;
+#[cfg(feature = "gecko")] use crate::gecko_bindings::structs::NonCustomCSSPropertyId;
 use crate::properties::{
     longhands::{
         self, visibility::computed_value::T as Visibility,
@@ -26,6 +26,7 @@ use std::mem;
 use rustc_hash::FxHashMap;
 use super::ComputedValues;
 use crate::properties::OwnedPropertyDeclarationId;
+use crate::dom::AttributeProvider;
 use crate::values::animated::{Animate, Procedure, ToAnimatedValue, ToAnimatedZero};
 use crate::values::animated::effects::AnimatedFilter;
 #[cfg(feature = "gecko")] use crate::values::computed::TransitionProperty;
@@ -36,12 +37,12 @@ use crate::values::generics::effects::Filter;
 use void::{self, Void};
 use crate::properties_and_values::value::CustomAnimatedValue;
 
-/// Convert nsCSSPropertyID to TransitionProperty
+/// Convert NonCustomCSSPropertyId to TransitionProperty
 #[cfg(feature = "gecko")]
 #[allow(non_upper_case_globals)]
-impl From<nsCSSPropertyID> for TransitionProperty {
-    fn from(property: nsCSSPropertyID) -> TransitionProperty {
-        TransitionProperty::NonCustom(NonCustomPropertyId::from_nscsspropertyid(property).unwrap())
+impl From<NonCustomCSSPropertyId> for TransitionProperty {
+    fn from(property: NonCustomCSSPropertyId) -> TransitionProperty {
+        TransitionProperty::NonCustom(NonCustomPropertyId::from_noncustomcsspropertyid(property).unwrap())
     }
 }
 
@@ -179,7 +180,7 @@ impl PartialEq for AnimationValue {
 impl AnimationValue {
     /// Returns the longhand id this animated value corresponds to.
     #[inline]
-    pub fn id(&self) -> PropertyDeclarationId {
+    pub fn id(&self) -> PropertyDeclarationId<'_> {
         if let AnimationValue::Custom(animated_value) = self {
             return PropertyDeclarationId::Custom(&animated_value.name);
         }
@@ -253,6 +254,7 @@ impl AnimationValue {
         context: &mut Context,
         style: &ComputedValues,
         initial: &ComputedValues,
+        attr_provider: &dyn AttributeProvider,
     ) -> Option<Self> {
         use super::PropertyDeclarationVariantRepr;
 
@@ -375,6 +377,7 @@ impl AnimationValue {
                         context.builder.stylist.unwrap(),
                         context,
                         &mut cache,
+                        attr_provider,
                     )
                 };
                 return AnimationValue::from_declaration(
@@ -382,6 +385,7 @@ impl AnimationValue {
                     context,
                     style,
                     initial,
+                    attr_provider,
                 )
             },
             PropertyDeclaration::Custom(ref declaration) => {
@@ -389,6 +393,7 @@ impl AnimationValue {
                     declaration,
                     context,
                     initial,
+                    attr_provider
                 )?)
             },
             _ => return None // non animatable properties will get included because of shorthands. ignore.
@@ -695,7 +700,7 @@ impl Animate for AnimatedFilter {
             % endfor
             % for func in ['Brightness', 'Contrast', 'Opacity', 'Saturate']:
             (&Filter::${func}(this), &Filter::${func}(other)) => {
-                Ok(Filter::${func}(animate_multiplicative_factor(this, other, procedure)?))
+                Ok(Filter::${func}(animate_multiplicative_factor(this.0, other.0, procedure)?.into()))
             },
             % endfor
             _ => Err(()),
@@ -711,7 +716,7 @@ impl ToAnimatedZero for AnimatedFilter {
             Filter::${func}(ref this) => Ok(Filter::${func}(this.to_animated_zero()?)),
             % endfor
             % for func in ['Brightness', 'Contrast', 'Opacity', 'Saturate']:
-            Filter::${func}(_) => Ok(Filter::${func}(1.)),
+            Filter::${func}(_) => Ok(Filter::${func}(1.0.into())),
             % endfor
             _ => Err(()),
         }
