@@ -24,19 +24,26 @@
 #define MACOS_VERSION_10_15_HEX 0x000A0F00
 #define MACOS_VERSION_10_16_HEX 0x000A1000
 #define MACOS_VERSION_11_0_HEX 0x000B0000
+#define MACOS_VERSION_12_0_HEX 0x000C0000
+#define MACOS_VERSION_13_0_HEX 0x000D0000
+#define MACOS_VERSION_14_0_HEX 0x000E0000
+#define MACOS_VERSION_26_0_HEX 0x001A0000
 
 #include "nsCocoaFeatures.h"
 #include "nsCocoaUtils.h"
 #include "nsDebug.h"
 #include "nsObjCExceptions.h"
 
+#include <atomic>
 #import <Cocoa/Cocoa.h>
 #include <sys/sysctl.h>
 
-/*static*/ int32_t nsCocoaFeatures::mOSVersion = 0;
+// The lazily-initialized version. 0 before initialization, non-zero after.
+static std::atomic<int32_t> sOSVersion = 0;
 
 // This should not be called with unchecked aMajor, which should be >= 10.
-inline int32_t AssembleVersion(int32_t aMajor, int32_t aMinor, int32_t aBugFix) {
+inline int32_t AssembleVersion(int32_t aMajor, int32_t aMinor,
+                               int32_t aBugFix) {
   MOZ_ASSERT(aMajor >= 10);
   return (aMajor << 16) + (aMinor << 8) + aBugFix;
 }
@@ -63,11 +70,13 @@ static int intAtStringIndex(NSArray* array, int index) {
 void nsCocoaFeatures::GetSystemVersion(int& major, int& minor, int& bugfix) {
   major = minor = bugfix = 0;
 
-  NSString* versionString = [[NSDictionary
-      dictionaryWithContentsOfFile:@"/System/Library/CoreServices/SystemVersion.plist"]
-      objectForKey:@"ProductVersion"];
+  NSString* versionString =
+      [[NSDictionary dictionaryWithContentsOfFile:
+                         @"/System/Library/CoreServices/SystemVersion.plist"]
+          objectForKey:@"ProductVersion"];
   if (!versionString) {
-    NS_ERROR("Couldn't read /System/Library/CoreServices/SystemVersion.plist to determine macOS "
+    NS_ERROR("Couldn't read /System/Library/CoreServices/SystemVersion.plist "
+             "to determine macOS "
              "version.");
     return;
   }
@@ -84,7 +93,8 @@ void nsCocoaFeatures::GetSystemVersion(int& major, int& minor, int& bugfix) {
   }
 }
 
-int32_t nsCocoaFeatures::GetVersion(int32_t aMajor, int32_t aMinor, int32_t aBugFix) {
+int32_t nsCocoaFeatures::GetVersion(int32_t aMajor, int32_t aMinor,
+                                    int32_t aBugFix) {
   int32_t macOSVersion;
   if (aMajor < 10) {
     aMajor = 10;
@@ -109,8 +119,8 @@ int32_t nsCocoaFeatures::GetVersion(int32_t aMajor, int32_t aMinor, int32_t aBug
   return macOSVersion;
 }
 
-/*static*/ void nsCocoaFeatures::InitializeVersionNumbers() {
-  NS_OBJC_BEGIN_TRY_IGNORE_BLOCK;
+/*static*/ int32_t nsCocoaFeatures::ComputeVersion() {
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
 
   // Provide an autorelease pool to avoid leaking Cocoa objects,
   // as this gets called before the main autorelease pool is in place.
@@ -118,19 +128,25 @@ int32_t nsCocoaFeatures::GetVersion(int32_t aMajor, int32_t aMinor, int32_t aBug
 
   int major, minor, bugfix;
   GetSystemVersion(major, minor, bugfix);
-  mOSVersion = GetVersion(major, minor, bugfix);
+  return GetVersion(major, minor, bugfix);
 
-  NS_OBJC_END_TRY_IGNORE_BLOCK;
+  NS_OBJC_END_TRY_BLOCK_RETURN(0);
 }
 
 /* static */ int32_t nsCocoaFeatures::macOSVersion() {
-  // Don't let this be called while we're first setting the value...
-  MOZ_ASSERT((mOSVersion & MACOS_VERSION_MASK) >= 0);
-  if (!mOSVersion) {
-    mOSVersion = -1;
-    InitializeVersionNumbers();
+  int32_t version = sOSVersion.load(std::memory_order_relaxed);
+  if (version != 0) {
+    return version;
   }
-  return mOSVersion;
+
+  // Compute the version. Multiple threads might do the computation
+  // concurrently. That's ok; they will all compute the same value.
+  version = ComputeVersion();
+  if (version != 0) {
+    sOSVersion.store(version, std::memory_order_relaxed);
+  }
+
+  return version;
 }
 
 /* static */ int32_t nsCocoaFeatures::macOSVersionMajor() {
@@ -145,33 +161,47 @@ int32_t nsCocoaFeatures::GetVersion(int32_t aMajor, int32_t aMinor, int32_t aBug
   return ExtractBugFixVersion(macOSVersion());
 }
 
-/* static */ bool nsCocoaFeatures::OnSierraExactly() {
-  return (macOSVersion() >= MACOS_VERSION_10_12_HEX) && (macOSVersion() < MACOS_VERSION_10_13_HEX);
-}
-
-/* Version of OnSierraExactly as global function callable from cairo & skia */
-bool Gecko_OnSierraExactly() { return nsCocoaFeatures::OnSierraExactly(); }
-
-/* static */ bool nsCocoaFeatures::OnHighSierraOrLater() {
-  return (macOSVersion() >= MACOS_VERSION_10_13_HEX);
-}
-
-/* static */ bool nsCocoaFeatures::OnMojaveOrLater() {
-  return (macOSVersion() >= MACOS_VERSION_10_14_HEX);
-}
-
-/* static */ bool nsCocoaFeatures::OnCatalinaOrLater() {
-  return (macOSVersion() >= MACOS_VERSION_10_15_HEX);
-}
-
 /* static */ bool nsCocoaFeatures::OnBigSurOrLater() {
-  // Account for the version being 10.16 (which occurs when the
-  // application is linked with an older SDK) or 11.0 on Big Sur.
+  // Account for the version being 10.16 or 11.0 on Big Sur.
+  // The version is reported as 10.16 if SYSTEM_VERSION_COMPAT is set to 1,
+  // or if SYSTEM_VERSION_COMPAT is not set and the application is linked
+  // with a pre-Big Sur SDK.
+  // Firefox sets SYSTEM_VERSION_COMPAT to 0 in its Info.plist, so it'll
+  // usually see the correct 11.* version, despite being linked against an
+  // old SDK. However, it still sees the 10.16 compatibility version when
+  // launched from the command line, see bug 1727624. (This only applies to
+  // the Intel build - the arm64 build is linked against a Big Sur SDK and
+  // always sees the correct version.)
   return ((macOSVersion() >= MACOS_VERSION_10_16_HEX) ||
           (macOSVersion() >= MACOS_VERSION_11_0_HEX));
 }
 
-/* static */ bool nsCocoaFeatures::IsAtLeastVersion(int32_t aMajor, int32_t aMinor,
+/* static */ bool nsCocoaFeatures::OnMontereyOrLater() {
+  // This check only works if SYSTEM_VERSION_COMPAT is off, otherwise
+  // Monterey pretends to be 10.16 and is indistinguishable from Big Sur.
+  // In practice, this means that an Intel Firefox build can return false
+  // from this function if it's launched from the command line, see bug 1727624.
+  // This will not be an issue anymore once we link against the Big Sur SDK.
+  return (macOSVersion() >= MACOS_VERSION_12_0_HEX);
+}
+
+/* static */ bool nsCocoaFeatures::OnVenturaOrLater() {
+  // See comments above regarding SYSTEM_VERSION_COMPAT.
+  return (macOSVersion() >= MACOS_VERSION_13_0_HEX);
+}
+
+/* static */ bool nsCocoaFeatures::OnSonomaOrLater() {
+  // See comments above regarding SYSTEM_VERSION_COMPAT.
+  return (macOSVersion() >= MACOS_VERSION_14_0_HEX);
+}
+
+/* static */ bool nsCocoaFeatures::OnTahoeOrLater() {
+  // See comments above regarding SYSTEM_VERSION_COMPAT.
+  return (macOSVersion() >= MACOS_VERSION_26_0_HEX);
+}
+
+/* static */ bool nsCocoaFeatures::IsAtLeastVersion(int32_t aMajor,
+                                                    int32_t aMinor,
                                                     int32_t aBugFix) {
   return macOSVersion() >= GetVersion(aMajor, aMinor, aBugFix);
 }
